#!/usr/bin/perl
#---------------------------------------------------------------------
# ProScan Update Manager CGIモジュール
#---------------------------------------------------------------------
# 管理機能
#---------------------------------------------------------------------
# Script name: manager.cgi
# Create date: 2005/10/11(V1.0)
# Auther:       T.Oyamada
#---------------------------------------------------------------------
# Change:
# 2013.08.07 activation時のホスト名を名前で指定した場合のIPアドレス変換
# 2013.08.22 managerのIPアドレスを変数として指定（ゆくゆくは外部ファイル化）
# 2019.02.22 htpasswdのオプション追加（CRYPTタイプ固定）
# 2019.10.11 UpdateOnlyパラメータのコンフィグ編集可能に
# 2020.11.02 x86ライセンス関係を削除
# 2021.11.23 apache2.4系でのuse encodingのエラー削除
# 2022.01.28 CacheCheckパラメータのコンフィグ編集可能に
#---------------------------------------------------------------------
# All Rights Reserved, Copyright (c) Promark Inc. 2005-2022
#---------------------------------------------------------------------
use strict;
use utf8;
use CGI;
#use CGI::Session;
use CGI::Carp qw(fatalsToBrowser);
use Time::Local;
use Fcntl;
use AnyDBM_File;
use Socket;
use Encode;
#use encoding "utf8";
#use encoding "euc-jp";

my $debug;
my $manager_ipaddr = "192.168.100.4";

my $psmt_version = "2.0 - build 3";
my $psmt_copyright = "All Rights Reserved, Copyright &copy; Promark Inc. 2005-2022";

# 管理データ格納ディレクトリを指定します。verchk.cgiで指定したものと
# 同じディレクトリとなります。
my $manager_dir = "/var/www/manager";

# wgetコマンドのパスを指定します。
my $wget = "/usr/bin/wget";

# PSMTのタイプ
my $master_flag = 1;
my $latest_ver_file = "";

# PSMT表示設定
my $psmt_title = "";
my $psmt_color = "#cccccc";
my $psmt_color2 = "#f0f0f0";

# 最新バージョン管理サーバのアドレスを３つ指定します。
# 通常は以下のサーバで問題ありません。
my $server1 = "clamav-latest.promark-inc.com";
my $server2 = "clamav-latest-2.promark-inc.com";
my $server3 = "update.promark-inc.com";

# Update用データ格納ディレクトリ
my $update_data_dir = "$manager_dir/update";

# アクセス許可端末
my $accept = ".";

# リスト表示時の1ページの行数
my $number_of_page = 10;

my $data_dir = "$manager_dir/data";
my $data_file = "$data_dir/client_db";
my $acl_file = "$data_dir/acl";
my $passwd_file = "$data_dir/passwd";
my $user_db_file = "$data_dir/users_db";
my $config_db_file = "$data_dir/config_db";
my $session_db_file = "$data_dir/session_db";
my $group_db_file = "$data_dir/group_db";
my $apply_db_file = "$data_dir/apply_db";
my $update_list_file = "$data_dir/update_list.csv";
my $scan_list_file = "$data_dir/scan_list.csv";
my $target_list_file = "$data_dir/target_list.txt";
my $csv_data_file = "$data_dir/client_data.csv";
my $command_dir = "$manager_dir/bin";
my $mirror_dir = "$manager_dir/mirror";
my $config_dir = "$data_dir/config";
my $license_dir = "$data_dir/license";
my $download_dir = "$data_dir/download";
#my $license_file_x86 = "$license_dir/proscan_i386.key";
my $license_file_s390 = "$license_dir/proscan_s390.key";
my $temp = "/tmp/proscan_version.txt";
my $latest = "$update_data_dir/latest/VERSION";
my $dist = "$update_data_dir/dist/VERSION";
my $bak0 = "$update_data_dir/backup0/VERSION";
my $bak1 = "$update_data_dir/backup1/VERSION";
my $bak2 = "$update_data_dir/backup2/VERSION";
my $active_command = "$command_dir/activate.sh";
my $setup_command = "$command_dir/setup.sh";
my $all_command = "$command_dir/all_clients.pl";
my $manager_command = "$command_dir/manager";
my $show_license_command = "$command_dir/showlic.pl";
my $proxy_conf_file = "$mirror_dir/proxy.conf";

my $cgi = "manager.cgi";

my $query = new CGI;
my $mode = $query->param('mode');
$mode = 'list' unless $mode;
my $page = $query->param('page');
my $order;
my $updown;
my $selecting_flag;
my $host_sel;
my $group_sel;
my $ver_stat_sel;
my $proscan_sel;
my $engine_sel;
my $vdf_sel;
my $update_sel;
my $scan_stat_sel;
my $scan_date_sel;

if($mode eq 'list' || $mode eq 'select_act') {
    $order = $query->param('order');
    $order = 0 if $order eq "";
    $updown = $query->param('updown');
    $updown = 'up' unless $updown;
    $selecting_flag = $query->param('sflag');
    $selecting_flag = 'off' unless $selecting_flag;
    # selecting
    $host_sel = $query->param('host_sel');
    $group_sel = $query->param('group_sel');
    $ver_stat_sel = $query->param('ver_stat_sel');
    $proscan_sel = $query->param('proscan_sel');
    $engine_sel = $query->param('engine_sel');
    $vdf_sel = $query->param('vdf_sel');
    $update_sel = $query->param('update_sel');
    $scan_stat_sel = $query->param('scan_stat_sel');
    $scan_date_sel = $query->param('scan_date_sel');
}

my $rc;
my $h;
my %verstatsel_val;

$verstatsel_val{'green'} = 0;
$verstatsel_val{'yellow'} = 1;
$verstatsel_val{'red'} = 2;

# 最新バージョン情報取得（アップデートサーバまたはファイル）
if ( $master_flag == 1 ) {
    unlink($temp);
    my $proxy = "";
    # check proxy config
    if ( -f $proxy_conf_file ) {
	my $option = `cat $proxy_conf_file`;
	my @opt = split(/ +/,$option);
	foreach (@opt) {
	    my ($key,$val) = /--(.*)=(.*)/;
	    if ( $key eq "proxy-host" ) {
		$proxy = "http://$val";
	    } elsif ( $key eq "proxy-port" ) {
		$proxy .= ":$val";
	    } elsif ( $key eq "proxy-user" ) {
		$wget .= " --proxy-user=$val";
	    } elsif ( $key eq "proxy-password" ) {
		$wget .= " --proxy-password=$val";
	    }
  	}
	$ENV{'http_proxy'} = $proxy;
	$debug = "use proxy: $proxy : $wget";
    }
    $rc = system("$wget -q -T 5 -O $temp http://$server1/latest_version3.txt");
    $rc = system("$wget -q -T 5 -O $temp http://$server2/latest_version3.txt") if $rc;
    $rc = system("$wget -q -T 5 -O $temp http://$server3/latest_version3.txt") if $rc;
} else {
    $temp = $latest_ver_file;
}

# クライアントチェック
my $ip = $ENV{'REMOTE_ADDR'};
my $accept = "";
if ( -f $acl_file ) {
    my @acl;
    open(ACL,$acl_file);
    while(<ACL>) {
        s/\x0D|\x0A|\x0D\x0A//;
        next if $_ eq "" | /^#/;
        push(@acl,$_);
    }
    close(ACL);
    $accept = join("|",@acl);
}
my $ok = 0;
$ok = 1 if ( $ip =~ /$accept/ );
my $acl;
if ( $ok == 1 ) {
    $acl = "allow";
} else {
    $acl = "deny";
}

# ACLチェック
if ( $acl eq 'deny' ) {
    &error("ACLエラー","接続が許可されていません。");
}

my $id;
my $pass;
my $oldpass;
my $pass_conf;
my $new_session;
my $session;

# パスワードチェック
if ( $mode eq 'login' ) {
    $id = $query->param('id');
    $pass = $query->param('pass');
    if( &CheckPassword($id,$pass,0) ) {
        &error("認証エラー","IDまたはパスワードが違います。");
    }
    $new_session = &regist_session($ip,$id,$pass);
    $mode = 'list';
} elsif ( $mode eq 'pass_chg' ) {
    # パスワード変更
    $session = &GetSessionID;
    if ( $session eq "" ) {
        &error("session undefined");
    }
    $id = $query->param('id');
    $pass = $query->param('pass');
    $pass_conf = $query->param('pass_conf');
    if($pass eq "" || $pass_conf eq "") {
        &error("パスワード未入力","パスワードが未入力です。必ず入力して下さい。");
    }
    if($pass ne $pass_conf) {
        &error("パスワード不一致","パスワードが一致しません。再度、入力しなおして下さい。");
    }
    my $new_session = &regist_session($ip,$id,$pass);
    &regist_new_passwd($new_session,$id,$pass);
    $mode = 'list';
} elsif ( $mode eq 'pass_chg2' ) {
    # パスワード変更
    $id = $query->param('id');
    $oldpass = $query->param('oldpass');
    if( &CheckPassword($id,$oldpass,1) ) {
        &error("パスワードエラー","現在のパスワードが正しくありません。");
    }
    $pass = $query->param('pass');
    $pass_conf = $query->param('pass_conf');
    if($pass eq "" || $pass_conf eq "") {
        &error("新パスワード未入力","新しいパスワードが未入力です。必ず入力して下さい。");
    }
    if($pass ne $pass_conf) {
        &error("新パスワード不一致","新しいパスワードが一致しません。再度、入力しなおして下さい。");
    }
    $new_session = &regist_session($ip,$id,$pass);
    &regist_new_passwd($new_session,$id,$pass);
    $mode = 'list';
} else {
    # セッション情報取得（Cookieより）
    $session = &GetSessionID;
    
    # もし、セッション情報がなければ認証ページ表示
    if( $session eq "" ) {
        &login_page;
        exit(0);
    }
    
    # パスワード変更
    if ( $mode eq 'pass_chg' ) {
        $id = $query->param('id');
        $oldpass = $query->param('oldpass');
        if( &CheckPassword($id,$oldpass,1) ) {
            &error("パスワードエラー","現在のパスワードが正しくありません。");
        }
        $pass = $query->param('pass');
        $pass_conf = $query->param('pass_conf');
        if($pass eq "" || $pass_conf eq "") {
            &error("パスワード未入力","パスワードが未入力です。必ず入力して下さい。");
        }
        if($pass ne $pass_conf) {
            &error("パスワード不一致","パスワードが一致しません。再度、入力しなおして下さい。");
        }
        &regist_new_passwd($session,$id,$pass);
    } else {
        # セッションIDからID,PASSを取得認証
        ($id,$pass) = &GetIDPASS($session);
    }
    
    # IDがなければログインページを開く
    if ( $id eq "" ) {
        &login_page;
        exit(0);
    }
    
    # 認証できなければ、エラー表示
    if( &CheckPassword($id,$pass,0) ) {
        &error("認証エラー","IDまたはパスワードが違います。($id,$session)");
    }
}

# 現在時刻取得
my @T = localtime;
my $date = sprintf("%04d/%02d/%02d",$T[5]+1900,$T[4]+1,$T[3]);
my $time = sprintf("%02d:%02d:%02d",$T[2],$T[1],$T[0]);

# 管理者権限
my $useradmin;
if ( $id eq 'admin' ) {
    $useradmin = qq(<input type="button" value="ユーザ管理" onClick="SubmitAction(this.form,'user_f')">);
} else {
    $useradmin = qq(<input type="button" value="パスワード変更" onClick="SubmitAction(this.form,'passwd_f')">);
}
    
# 処理振り分け
if ( $mode eq 'select_act' ) {
        &select_action;
} elsif ( $mode eq 'config_act' ) {
        &config_action;
} elsif ( $mode eq 'user_act' ) {
        &user_action;
} elsif ( $mode eq 'group_act' ) {
        &group_action;
} elsif ( $mode eq 'option_act' ) {
        &option_action;
} elsif ( $mode eq 'apply_act' ) {
        &apply_action;
} elsif ( $mode eq 'active' ) {
        &activation;
} elsif ( $mode eq 'license_up' ) {
        &license_upload;
} elsif ( $mode eq 'control_act' ) {
        &control_action;
#} elsif ( $mode eq 'rollback_act' ) {
#        &rollback_action;
} else {
    
    # HTML出力
    &header($new_session);
    if ( $mode eq 'list' ) {
        &list($page);
    } elsif ( $mode eq 'version_check' ) {
        &version_check;
    } elsif ( $mode eq 'verchk_log' ) {
        &version_check_log;
    } elsif ( $mode eq 'config_f' ) {
        &config_form($query->param('config'));
    } elsif ( $mode eq 'active_f' ) {
        &activation_form;
    } elsif ( $mode eq 'apply_f' ) {
        &apply_list_form;
    } elsif ( $mode eq 'user_f' ) {
        &user_list_form;
    } elsif ( $mode eq 'user_ef' ) {
        &user_entry_form;
    } elsif ( $mode eq 'passwd_f' ) {
        &password_change_form($id);
    } elsif ( $mode eq 'option_f' ) {
        &option_form;
    } elsif ( $mode eq 'license_f' ) {
        &license_form;
    } elsif ( $mode eq 'control_f' ) {
        &control_form;
    }
    &footer;
}

exit;

# HTMLヘッダ出力
sub header {
    my($session_data) = @_;
    my $multipart;

    # HTMLヘッダ出力
    print "Content-Type: text/html\n";
    if($session_data) {
        print qq(Set-Cookie: session=$session_data; path=/cgi-bin/\n);
    }
    print "\n";
    print "<html>\n";
    
    # Listの場合は、再読み込みさせる
    #print qq(<meta http-equiv="refresh" content="120">\n) if $mode eq 'list';
    
    # 共通ヘッダ出力
    print <<EOF;
<meta http-equiv="content-type" content="text/html;charset=UTF-8">
<head>
<title>ProScan Management Tool</title>
<style type="text/css">
A { text-decoration: none; }
.title { font-size: 18pt; font-weight: bold; }
.subtitle { font-size: 12pt; font-weight: bold; }
.updown { font-size: 8pt; text-align: right; }
.current_page { font-size: 16pt; font-weight: bold; padding: 3px; background-color: black; color: white; }
</style>
<!-- $mode -->
<script language="Javascript">
<!--
function SubmitAction(F,M) {
    F.mode.value = M;
    F.submit();
}
EOF
    
    # 各フォーム専用Javascript出力
    
    if ( $mode eq "list" ) {
        
        # List表示用Javascript
        
        print <<EOF;
function SubmitTest(F) {
    F.mode.value = "test_check";
    F.submit();
}
function SelectingAction(F) {
    F.mode.value = "select_act";
    F.submit();
}
function RollbackConfirm(F,h) {
    n = parseInt(h) + 1;
    if(window.confirm("ロールバックを実施します。よろしいですか？（No."+n+"）")) {
        //F.mode.value = "rollback_act";
        //F.rollback.value = h;
        //F.submit();
        URL = "/cgi-bin/result.cgi?type=rollback" + h;
        window.open(URL,"ShowCommandLog","scrollbars=yes,resizable=yes");
    }
}
function SubmitControl(F,C) {
    F.mode.value = "control_f";
    F.client.value = C;
    F.submit();
}
function SortOrder(Item) {
    F = document.manager;
    OldItem = F.order.value;
    F.order.value = Item;
    UpDown = F.updown.value;
    if(OldItem != Item) UpDown = '';
    //msg = 'item=' + Item + ',updown='+UpDown;
    if ( UpDown == '' ) UpDown = 'down';
    else if ( UpDown == 'down' ) UpDown = 'up';
    else if ( UpDown == 'up' ) UpDown = 'down';
    F.updown.value = UpDown;
    //msg += '-->'+UpDown;
    //window.alert(msg);
    F.mode.value = "list";
    F.submit();
}
function CommandExecute(T) {
    if ( T == "mirror" ) {
        com = "ミラーリング";
    } else if ( T == "latest2dist" ) {
        com = "最新版反映";
    }
    if(window.confirm(com+"コマンドを実行します。よろしいですか？")) {
        URL = "/cgi-bin/result.cgi?type=" + T;
        window.open(URL,"ShowCommandLog","scrollbars=yes,resizable=yes");
    }
}
function Refresh() {
    location.href = "/cgi-bin/manager.cgi";
}
function PageJump(P) {
    F = document.manager;
    F.mode.value = 'list';
    F.page.value = P;
    F.submit();
}
EOF

    } elsif ( $mode eq "user_f" || $mode eq "user_ef" ) {
    
        # ユーザ管理用フォームJavascript
        
        print <<EOF;
function SubmitUserForm(F,M,ID) {
    if ( M == 'user_new' ) {
        submode = 'add';
    } else if ( M == 'user_edit' ) {
        submode = 'mod';
        F.uid.value = ID;
    } else {
        submode = 'new';
    }
    F.mode.value = 'user_ef';
    F.submode.value = submode;
    F.submit();
}
function SubmitUserAction(F,M,ID) {
    F.mode.value = 'user_act';
    F.submode.value = M;
    if(ID != "") F.uid.value = ID;
    F.submit();
}
function UserDelConfirm(F,uid) {
    RES = window.confirm("ユーザ "+uid+" を削除します。よろしいですか？");
    if ( RES == true ) {
        SubmitUserAction(F,'del',uid);
    }
}
EOF
    } elsif ( $mode eq "option_f" || $mode eq "option_act" || $mode eq "group_act" ) {
    
        # オプション用フォームJavascript
        
        print <<EOF;
function ChangeGroup(F) {
    F.mode.value = "option_f";
    F.submit();
}
function SubmitGroupAction(F,M) {
    F.mode.value = 'group_act';
    F.submode.value = M;
    F.submit();
}
function MoveDefSide(F) {
    I = F.undef_group.selectedIndex;
    if ( I >= 0 ) {
        val = F.undef_group.options[I].value;
        txt = F.undef_group.options[I].text;
        F.def_group.options[F.def_group.length] = new Option(txt,val);
        F.undef_group.options[I] = null;
    }
}
function MoveUndefSide(F) {
    I = F.def_group.selectedIndex;
    if ( I >= 0 ) {
        val = F.def_group.options[I].value;
        txt = F.def_group.options[I].text;
        F.undef_group.options[F.undef_group.length] = new Option(txt,val);
        F.def_group.options[I] = null;
    }
}
function SubmitOptionAction(F) {
    F.mode.value = "option_act";
    L = F.def_group.length;
    dlist = new Array(L);
    for(i=0;i<L;i++) {
        dlist[i] = F.def_group.options[i].value;
    }
    F.def_list.value = dlist.join(",");
    L = F.undef_group.length;
    ulist = new Array(L);
    for(i=0;i<L;i++) {
        ulist[i] = F.undef_group.options[i].value;
    }
    F.undef_list.value = ulist.join(",");
    F.submit();
}
EOF
    } elsif ( $mode eq "license_f" || $mode eq "license_up" ) {
        
        # ライセンス管理フォーム用Javascript
        
        $multipart = qq(enctype="multipart/form-data");
        print <<EOF;
function UploadCheck(F) {
    if ( F.s390.value == "" && F.i386.value == "" ) {
        window.alert("ライセンスファイルを指定してください");
        return false;
    }
    if ( F.s390.value != "" ) {
        key_file = F.s390.value;
        if ( ! key_file.match(/PSHB08-/) ) {
            window.alert("zSeriesのライセンスが正しくありません");
            return false;
        }
    }
    //if (  F.i386.value != "" ) {
    //    key_file = F.i386.value;
    //    if ( ! key_file.match(/PSHB01-/) ) {
    //        window.alert("xSeriesのライセンスが正しくありません");
    //        return false;
    //    }
    //}
    F.mode.value = "license_up";
    document.getElementById('msg').innerHTML = '<img src="/loadinfo.gif">';
    F.submit();
}
EOF
    } elsif ( $mode eq "control_f" || $mode eq "control_act" ) {
        
        # コントロールフォーム用Javascript
        print <<EOF;
function ControlAction(F,M) {
    document.getElementById('msg').innerHTML = '<img src="/loadinfo.gif">';
    F.mode.value = 'control_act';
    F.submode.value = M;
    if ( M == "deactivate" ) {
        inp = window.prompt("管理対象から外します。パスワードを指定してください。","");
        F.passwd.value = inp;
    }
    F.submit();
}

function ScanLog(IP) {
    URL = "/cgi-bin/scanlog.cgi?ip=" + IP;
    window.open(URL,"ScanLog","scrollbars=yes,resizable=yes");
}

EOF
        $multipart = qq(enctype="multipart/form-data");
        
    } elsif ( $mode eq "result" ) {
        
        # 結果出力用Javascript
        $multipart = "";
        
    } else {
        
        # その他のフォーム用Javascript
        
        $multipart = "";
    }
    
    # 残りの共通ヘッダ出力
    
    print <<EOF;
function debug_print() {
    return;
    F = document.manager;
    if(F.mode.value == 'list') {
        msg = 'item='+F.order.value+',updown='+F.updown.value;
        window.status = msg;
    }
}
//-->
</script>
<!-- DEBUG: $debug -->
</head>
<body>
<form action="/cgi-bin/manager.cgi" method="POST" name="manager" $multipart>
<input type="hidden" name="mode" value="$mode">
EOF

    # hiddenタグ出力
    if ( $mode eq 'list') {
        print qq(<input type="hidden" name="page" value="$page">\n);
        print qq(<input type="hidden" name="client" value="">\n);
        print qq(<input type="hidden" name="rollback" value="">\n);
        print qq(<input type="hidden" name="order" value="$order">\n);
        print qq(<input type="hidden" name="updown" value="$updown">\n);
        print qq(<input type="hidden" name="sflag" value="$selecting_flag">\n);
    }
}

# HTMLフッタ出力
sub footer {
    print <<EOF;
</form>
</body>
</html>
EOF
}

sub list {
    my ($page) = @_;
    $page = 1 unless $page;
    
    my $version;
    my $promark_version;
    my $latest_version;
    my $date_time;
    my $proscan_latest;
    my $engine_latest;
    my $vdf_latest;
    my $d;
    my $t;
    my $dist_version;
    my $dist_update_time;
    my $proscan_dist;
    my $engine_dist;
    my $vdf_dist;
    my $bak0_version;
    my $bak0_update_time;
    my $button_bak0;
    my $proscan_bak0;
    my $engine_bak0;
    my $vdf_bak0;
    my $bak1_version;
    my $bak1_update_time;
    my $button_bak1;
    my $proscan_bak1;
    my $engine_bak1;
    my $vdf_bak1;
    my $bak2_version;
    my $bak2_update_time;
    my $button_bak2;
    my $proscan_bak2;
    my $engine_bak2;
    my $vdf_bak2;
    my $latest_status = "最新版です";

    # 最新バージョン情報
    if ( -f $temp ) {
        open(VER,"$temp");
        $version = <VER>;
        chomp($version);
        close(VER);
        ($proscan_latest,$engine_latest,$vdf_latest,$d,$t) = split(/,/,$version);
	$promark_version = "$proscan_latest,$engine_latest,$vdf_latest";
        $date_time = "$d $t";
    }
    if ( $version eq "" ) {
        $date_time = "取得できませんでした";
    }
    # PSMTとプロマークサーバとの違いを判定
    open(VER,"$latest");
    $version = <VER>;
    chomp($version);
    close(VER);
    my @ver = split(/,/,$version);
    $latest_version = "$ver[0],$ver[1],$ver[2]";
    if ( $promark_version ne $latest_version ) {
	$latest_status = "最新版ではありません";
    }
    
    # distribution status
    if ( -f $dist ) {
        open(VER,"$dist");
        $version = <VER>;
        chomp($version);
        close(VER);
        ($proscan_dist,$engine_dist,$vdf_dist,$d,$t) = split(/,/,$version);
	$dist_version = "$proscan_dist,$engine_dist,$vdf_dist";
        $dist_update_time = "$d $t";
    } else {
        $dist_version = "";
        $dist_update_time = "データがありません";
    }
    
    # backup status
    if ( -f $bak0 ) {
        open(VER,"$bak0");
        $version = <VER>;
        chomp($version);
        close(VER);
        ($proscan_bak0,$engine_bak0,$vdf_bak0,$d,$t) = split(/,/,$version);
  	$bak0_version = "$proscan_bak0,$engine_bak0,$vdf_bak0";
        $bak0_update_time = "$d $t";
        $button_bak0 = qq(<input type="button" value="ロールバック" onClick="RollbackConfirm(this.form,'0')">);
    }

    if ( -f $bak1 ) {
        open(VER,"$bak1");
        $version = <VER>;
        chomp($version);
        close(VER);
        ($proscan_bak1,$engine_bak1,$vdf_bak1,$d,$t) = split(/,/,$version);
  	$bak1_version = "$proscan_bak1,$engine_bak1,$vdf_bak1";
        $bak1_update_time = "$d $t";
        $button_bak1 = qq(<input type="button" value="ロールバック" onClick="RollbackConfirm(this.form,'1')">);
    }
    if ( -f $bak2 ) {
        open(VER,"$bak2");
        $version = <VER>;
        chomp($version);
        close(VER);
        ($proscan_bak2,$engine_bak2,$vdf_bak2,$d,$t) = split(/,/,$version);
  	$bak2_version = "$proscan_bak2,$engine_bak2,$vdf_bak2";
        $bak2_update_time = "$d $t";
        $button_bak2 = qq(<input type="button" value="ロールバック" onClick="RollbackConfirm(this.form,'2')">);
    }
    
    # group list
    tie( my %group_data, 'AnyDBM_File', $group_db_file, O_RDONLY, 0644 );
    my $group_select_tag = qq(<option value=""></option>\n);
    foreach my $g (sort keys %group_data) {
	my $selected = ($g eq $group_sel) ? "selected" : "";
        $group_select_tag .= qq(<option value="$g" $selected>$group_data{$g}</option>\n);
    }
    untie(%group_data);
    
    # sort
    my @sort_order;
    $sort_order[$order] = qq(<span class="updown">&nbsp;▲</span>) if $updown eq "up";
    $sort_order[$order] = qq(<span class="updown">&nbsp;▼</span>) if $updown eq 'down';
    
    print <<EOF;
<table width="100%">
<tr>
<td><span class="title">ProScan管理ツール$psmt_title</span></td><td align="right">ver. $psmt_version</td>
</tr>
</table>
<HR>
<input type="button" value="最新情報を取得" onClick="SubmitAction(this.form,'version_check')">
<input type="button" value="コンフィグ編集" onClick="SubmitAction(this.form,'config_f')">
<input type="button" value="アクティベーション" onClick="SubmitAction(this.form,'active_f')">
<!--input type="button" value="一括順序管理" onClick="SubmitAction(this.form,'apply_f')"-->
<input type="button" value="オプション" onClick="SubmitAction(this.form,'option_f')">
<input type="button" value="ライセンス管理" onClick="SubmitAction(this.form,'license_f')">
$useradmin
<input type="button" value="再表示" onClick="Refresh()">
<hr>
<table cellspasing=0 cellpadding=0>
<tr>
<td>
  【ProScan最新情報】<br>
  <table cellspacing=1 cellpadding=0 bgcolor=#000000>
  <tr><td>
  <table cellspacing=1 cellpadding=4 bgcolor=#000000>
  <tr bgcolor="$psmt_color2">
  <td>ProScan Version</td><td>Engine Version</td><td>VDF version</td><td>Latest</td><td>Action</td><td>PSMT Status</td>
  </tr>
  <tr bgcolor=#ffffff>
  <th>$proscan_latest<br></th><th>$engine_latest</th><th>$vdf_latest</th><td>$date_time</td><td><input type="button" value="ミラーリング" onClick="CommandExecute('mirror')"></td><td>$latest_status</td>
  </tr>
  </table>
  </td></tr>
  </table>
</td>
</tr>
</table>
<hr>
<table cellspasing=0 cellpadding=0>
<tr>
<td>
  【本番配布用マスタ情報】<br>
  <table cellspacing=1 cellpadding=0 bgcolor=#000000>
  <tr><td>
  <table cellspacing=1 cellpadding=4 bgcolor=#000000>
  <tr bgcolor="$psmt_color2">
  <td>ProScan Version</td><td>Engine Version</td><td>VDF version</td><td width=120>Latest</td><td>Action</td>
  </tr>
  <tr bgcolor=#ffffff>
  <th>$proscan_dist<br></th><th>$engine_dist<br></th><th>$vdf_dist<br></th><td>$dist_update_time<br></td><td><input type="button" value="最新版反映" onClick="CommandExecute('latest2dist')"></td>
  </tr>
  </table>
  </td></tr>
  </table>
</td>
<td><img src="/spacer.gif" width=40></td>
<td>
  【バックアップ・マスタ情報】<br>
  <table cellspacing=1 cellpadding=0 bgcolor=#000000>
  <tr><td>
  <table cellspacing=1 cellpadding=4 bgcolor=#000000>
  <tr bgcolor="$psmt_color2">
  <td width=100>Action<br></td><td>No.</td><td>ProScan Version</td><td>Engine Version</td><td>VDF version</td><td width=120>Latest</td>
  </tr>
  <tr bgcolor=#ffffff>
  <td>$button_bak0<br></td><td align="right">1</td><th>$proscan_bak0<br></th><th>$engine_bak0<br></th><th>$vdf_bak0<br></th><td>$bak0_update_time<br></td>
  </tr>
  <tr bgcolor=#ffffff>
  <td>$button_bak1<br></td><td align="right">2</td><th>$proscan_bak1<br></th><th>$engine_bak1<br></th><th>$vdf_bak1<br></th><td>$bak1_update_time<br></td>
  </tr>
  <tr bgcolor=#ffffff>
  <td>$button_bak2<br></td><td align="right">3</td><th>$proscan_bak2<br></th><th>$engine_bak2<br></th><th>$vdf_bak2<br></th><td>$bak2_update_time<br></td>
  </tr>
  </table>
  </td></tr>
  </table>
</td>
</tr>
</table>
<hr>
【管理サーバ一覧】　　LastUpdate:$date $time<br>
<table cellspacing=1 cellpadding=0 bgcolor=#000000>
<tr><td>
<table cellspacing=1 cellpadding=4 bgcolor=#000000>

<!-- header -->
<tr bgcolor="$psmt_color2">
<td align=center>No.</td>
<td><a href="javascript:SortOrder(0)">Host</a>$sort_order[0]</td>
<td><a href="javascript:SortOrder(1)">Group</a>$sort_order[1]</td>
<td><a href="javascript:SortOrder(2)">Status</a>$sort_order[2]</td>
<td><a href="javascript:SortOrder(3)">ProScan version</a>$sort_order[3]</td>
<td><a href="javascript:SortOrder(4)">Engine version</a>$sort_order[4]</td>
<td><a href="javascript:SortOrder(5)">VDF version</a>$sort_order[5]</td>
<td><a href="javascript:SortOrder(7)">Latest update</a>$sort_order[7]</td>
<td><a href="javascript:SortOrder(8)">Scan</a>$sort_order[8]</td>
<td><a href="javascript:SortOrder(9)">Last Scan Time</a>$sort_order[9]</td>
<td><input type="button" value="Control" onClick="SubmitControl(this.form,'$h')"><br></td>
</tr>

<tr bgcolor=#ffffff>
 <td align=center><br></td>
 <td align=center><input type="text" name="host_sel" value="$host_sel" size=16></td>
 <td align=center>
 <select name="group_sel">
$group_select_tag
 </select></td>
 <td align=center><select name="ver_stat_sel">
 <option value=""></option>
 <option value="green">正常</option>
 <option value="yellow">注意</option>
 <option value="red">異常</option>
 </select></td>
 <td align=center><input type="text" name="proscan_sel" value="$proscan_sel" size=16></td>
 <td align=center><input type="text" name="engine_sel" value="$engine_sel" size=16></td>
 <td align=center><input type="text" name="vdf_sel" value="$vdf_sel" size=16></td>
 <td align=center><input type="text" name="update_sel" value="$update_sel" size=16></td>
 <td align=center><select name="scan_stat_sel">
 <option value=""></option>
 <option value="green">正常</option>
 <option value="red">異常</option>
 </select></td>
 <td><input type="text" name="scan_date_sel" value="$scan_date_sel" size=16></td>
 <td><input type="button" value="絞込み" onClick="SelectingAction(this.form)"></td>
</tr>
EOF
    
    open(LIST,">$target_list_file");
    open(CSV,">$csv_data_file");
    
    tie( my %client_data, 'AnyDBM_File', $data_file, O_CREAT|O_RDWR, 0644 );

    my $n = 1;
    my $now = time;
    my $bp = 0;
    my $page_link = "";
    my $key;
    my %sort_order;

    foreach $h (sort keys %client_data) {
        
        # output csv
        print CSV "$h,$client_data{$h}\x0D\x0A";
        
        # listing
        my @D = split(/,/,$client_data{$h});
        my $status = &status_check($dist_version,$client_data{$h});

        # selecting
        if ( $selecting_flag eq "on" ) {
            if($host_sel) {
                next if $h !~ /$host_sel/;
            }
            if($group_sel) {
                next if $D[1] ne $group_sel;
            }
            if($ver_stat_sel) {
                next if $status != $verstatsel_val{$ver_stat_sel};
            }
            if($proscan_sel) {
                next if $D[4] !~ /$proscan_sel/;
            }
            if($engine_sel) {
                next if $D[6] !~ /$engine_sel/;
            }
            if($vdf_sel) {
                next if $D[9] !~ /$vdf_sel/;
            }
            if($update_sel) {
                next if $D[11] !~ /$update_sel/;
            }
            if($scan_stat_sel eq 'red') {
                next if $D[13] eq "0";
            }
            if($scan_stat_sel eq 'green') {
                next if $D[13] ne "0";
            }
            if($scan_date_sel) {
                next if $D[14] !~ /$scan_date_sel/;
            }
        }
        
        if ( $order == 0 ) {
            $key = $h;
        } elsif ( $order == 1 ) {
            $key = $D[1];
        } elsif ( $order == 2 ) {
            $key = $status;
        } elsif ( $order == 3 ) {
            $key = $D[4];
        } elsif ( $order == 4 ) {
            $key = $D[6];
        } elsif ( $order == 5 ) {
            $key = $D[9];
        } elsif ( $order == 6 ) {
            $key = $D[10];
        } elsif ( $order == 7 ) {
            $key = $D[11];
        } elsif ( $order == 8 ) {
            $key = $D[13];
        } elsif ( $order == 9 ) {
            $key = $D[14];
        }
        
        push(@{$sort_order{$key}},$h);
        #print "<!-- ORDER: $key $h -->\n";
    }
    
    my @list_data = ();
    if ( $updown eq 'up' ) {
        foreach (sort {$a cmp $b} keys %sort_order) {
            push(@list_data,@{$sort_order{$_}});
        }
    } else {
        foreach (sort {$b cmp $a} keys %sort_order) {
            push(@list_data,@{$sort_order{$_}});
        }
    }
    
    my $color;
    my @D;
    my $status;
    my $p;
    my $bp;
    my $status_icon;
    my $result_icon;

    foreach $h (@list_data) {
        if ( $selecting_flag eq "on" ) {
            $color = "#ffffe0";
        } else {
            $color = "#ffffff";
        }
        # listing
        @D = split(/,/,$client_data{$h});
        $status = &status_check($dist_version,$client_data{$h});
        print LIST "$h\n";
        $p = int(($n-1)/$number_of_page)+1;
        if ( $p == $page ) {
            if ( $p != $bp ) {
		$page_link .= "｜" if $page_link;
                $page_link .= qq(<span class="current_page">$p</span>);
                $bp = $p;
            }
            
            $status_icon = "<font color=green>●</font>";
            $status_icon = "<font color=yellow>●</font>" if $status == 1;
            $status_icon = "<font color=red>●</font>" if $status == 2;
            
            $result_icon = "<font color=green>●</font>";
            $result_icon = "<font color=yellow>●</font>" if $D[13] eq "ERR" || $D[13] eq "";
            $result_icon = "<font color=red>●</font>" if $D[13] >= 1;
            
            print qq(<tr bgcolor="$color">);
            print "<td align=center>$n</td>";
            print "<td align=center>$h<br>($D[0])</td>";
            print "<td align=center>$D[1]</td>";
            print "<td align=center>$status_icon</td>";
            print "<td align=center>$D[4]</td>";
            print "<td align=center>$D[6]</td>";
            print "<td align=center>$D[9]</td>";
            $D[11] =~ s/ /<br>/;
            print "<td align=center>$D[11]</td>";
            print "<td align=center>$result_icon</td>";
            $D[14] =~ s/ /<br>/;
            print "<td align=center>$D[14]</td>";
            print qq(<td align=center><input type="button" value="control" onClick="SubmitControl(this.form,'$h')"></td>);
            print "</tr>\n";
        } else {
            if ( $p != $bp ) {
		$page_link .= "｜" if $page_link;
                $page_link .= qq(<a href="javascript:PageJump($p)">$p</a>);
                $bp = $p;
            }
        }
        $n++;
    }
    
    close(LIST);
    close(CSV);
    
    # page link
    my $link = "";
    if ( $n == 1 ) {
        print qq(<tr bgcolor="#ffffff" height=30><th colspan="12">一致するクライアントは見つかりませんでした<br></th></tr>\n);
    } elsif ( $bp > 1 ) {
        $link = <<EOF;
<table align=center>
<tr><td>
<a href="javascript:PageJump(1)">≪</a>&nbsp;
$page_link
<a href="javascript:PageJump($bp)">&nbsp≫</a>
</td></tr>
</table>
EOF
    }

    print <<EOF;
</table>
</td></tr>
</table>
$link
<hr>
<input type="hidden" name="target_list" value="$target_list_file">
<div align=right>$psmt_copyright</div>
EOF
#&client_dump;
}

sub vchl {
    my($v1,$v2) = @_;
    $v1 = sprintf("%d%03d%03d%03d",split(/\./,$v1));
    $v2 = sprintf("%d%03d%03d%03d",split(/\./,$v2));
    return ($v1 - $v2);
}

sub status_check {
    my($latest,$current) = @_;
    return 2 unless $latest;
    my($proscan,$engine,$vdf) = split(/,/,$latest);
    my @D = split(/,/,$current);
    return 2 if "$D[4]$D[6]$D[9]" eq "";
    my $cp1 = vchl($proscan,$D[4]);
    my $cp2 = vchl($engine,$D[6]);
    my $cp3 = vchl($vdf,$D[9]);
    
    return 0 if $cp1<=0 && $cp2<=0 && $cp3<=0;
    return 1 if $cp1>0 || $cp2>0 || $cp3>0;
    return 2 if $cp1>0 && $cp2>0 && $cp3>0;
}

sub error {
    my($mesg,$desc) = @_;
    print <<EOF;
Content-Type: text/html
Set-Cookie: session=$session; path=/cgi-bin;  expires=Fri, 31-Dec-1999 23:59:59 GMT;

<html>
<meta http-equiv="content-type" content="text/html;charset=UTF-8">
<head>
<title>PSMTエラー</title>
</head>
<!-- $session -->
<body>
<div align="center">
<h2>PSMT エラー</h2>
</div>
<hr>
<div align="center">
<table bgcolor="#ffcccc" cellpadding=20 width="400" height="100">
<tr><td style="background-color: #ffcccc">
<table cellpadding=5 bgcolor="#ffcccc">
<tr><td style="background-color: #ffcccc"><font color="red"><b>$mesg</b></font></td></tr>
<tr><td style="background-color: #ffcccc">$desc</td></tr>
</table>
</td></tr>
</table>
EOF
&db_dump('session');

    print <<EOF;
<form>
<br>
<input type="button" value="戻る" onClick="history.back()">
</div>
<p>
<hr>
<div align="center">$psmt_copyright</div>
</body>
</html>
EOF
    exit(0);
}

# CSV形式から値を取り出して配列を返す
sub csv2val {
    my($tmp) = @_;
    $tmp =~ s/(?:\x0D\x0A|[\x0D\x0A])?$/,/;
    map {/^"(.*)"$/ ? scalar($_ = $1, s/""/"/g, $_) : $_} ($tmp =~ /("[^"]*(?:""[^"]*)*"|[^,]*),/g);
}

# Cookie
sub GetCookie {
    my($COOKIE_DATA) = @_;
    
    my($cookie_pair,$key,$val);
    my @cookie_pairs = split(/; /,$ENV{'HTTP_COOKIE'});
    
    foreach $cookie_pair ( @cookie_pairs ) {
        ($key,$val) = split(/=/,$cookie_pair);
        $val =~ s/\+/ /g;
        $val =~ s/%([0-9a-fA-F][0-9a-fA-F])/pack("C",hex($1))/eg;
        $COOKIE_DATA->{$key} = $val;
    }
}

sub GetSessionID {
    my %cookie;
    &GetCookie(\%cookie);
    return $cookie{'session'};
}

sub regist_session {
    my($ip,$id,$pass) = @_;
    my $addr = inet_aton($ip);
    my $time = time;
    my($session) = sprintf("ss%x%d",$addr,$time);
    tie( my %session_list, 'AnyDBM_File', $session_db_file, O_CREAT|O_RDWR, 0644 );
    $session_list{$session} = "$ip,$id,$pass,$time";
    untie(%session_list);
    return $session;
}

sub regist_new_passwd {
    my($session,$id,$pass) = @_;
    
    # session change
    tie( my %session_list, 'AnyDBM_File', $session_db_file, O_CREAT|O_RDWR, 0644 );
    my($ip,$oid,$opass,$otime) = split(/,/,$session_list{$session});
    if ( $id ne $oid ) {
        &error("セッションエラー","セッションを継続できませんでした。セッション情報が合っていません。");
    }
    my $time = time;
    $session_list{$session} = "$ip,$id,$pass,$time";
    untie(%session_list);
    
    # new expire date & new passwd set
    tie( my %user_list, 'AnyDBM_File', $user_db_file, O_RDWR, 0600 );
    my @U = split(/,/,$user_list{$id});
    my $expire = $time + $U[1] * 86400;
    $expire = 0 unless $U[1];
    my $crypt_pass = crypt($pass,$pass);
    $user_list{$id} = "$crypt_pass,$U[1],$expire";
    untie( %user_list );
}

sub GetIDPASS {
    my($session) = @_;
    tie( my %session_list, 'AnyDBM_File', $session_db_file, O_CREAT|O_RDWR, 0644 );
    my($ip,$id,$pass,$time) = split(/,/,$session_list{$session});
    untie(%session_list);
    return ($id,$pass);
}

sub CheckPassword {
    my($id,$pass,$flag) = @_;
    
    tie( my %user_list, 'AnyDBM_File', $user_db_file, O_RDONLY, 0644 );
    my $user_entry = $user_list{$id};
    untie( %user_list );
    return 1 unless $user_entry;             # エントリーなし
    my($pw,$expire_days,$expire) = split(/,/,$user_entry);
    my $now = time;
    my ($salt) = $pw =~ /(..)/;
    my $crypt_pass = crypt($pass,$salt);
    #$crypt_pass = $pass;
    return 1 if $pw ne $crypt_pass;          # パスワード不一致
    return 0 if $flag == 1;                  # 有効期限チェックなし
    return 0 unless $expire_days;            # 制限なし
    &password_change($id) if $expire < $now; # 有効期限オーバ
    return 0;                                # 認証OK
}

sub login_page {
    print <<EOF;
Content-Type: text/html

<html>
<meta http-equiv="content-type" content="text/html;charset=UTF-8">
<head>
<title>PSMT Login</title>
</head>
<body>
<form action="/cgi-bin/$cgi" method="POST" name="login">
<input type="hidden" name="mode" value="login">
<div align="center">
<h2>PSMT ログイン$psmt_title</h2>
</div>
<hr>
<div align="center">
<table bgcolor="#ffffff" cellpadding=20 width="400" height="100">
<tr><td style="background-color: #ffffff">
<table cellpadding=5 bgcolor="$psmt_color" align="center">
<tr><td style="background-color: $psmt_color">ユーザID</td><td><input type="text" name="id" size=20></td></tr>
<tr><td style="background-color: $psmt_color">パスワード</td><td><input type="password" name="pass" size=20></td></tr>
</table>
</td></tr>
</table>
<br>
<input type="submit" value="ログイン">
<input type="button" value="キャンセル" onClick="history.back()">
</div>
<p>
<hr>
<div align="center">$psmt_copyright</div>
</body>
</html>
EOF
}

sub password_change {
    my($id) = @_;
    tie( my %user_list, 'AnyDBM_File', $user_db_file, O_RDONLY, 0644 );
    my @U = split(/,/,$user_list{$id});
    untie( %user_list );
    print <<EOF;
Content-Type: text/html

<html>
<meta http-equiv="content-type" content="text/html;charset=UTF-8">
<head>
<title>PSMT Password Chage</title>
</head>
<body>
<form action="/cgi-bin/manager.cgi" method="POST" name="pass_chg_form">
<input type="hidden" name="mode" value="pass_chg2">
<input type="hidden" name="id" value="$id">
<div align="center">
<h2>PSMT パスワード</h2>
</div>
<hr>
<div align="center">
パスワードの有効期限が切れています。<br>
パスワードの再設定をお願いします。<br>
<table bgcolor="#ffffff" cellpadding=20 width="400" height="100">
<tr><td style="background-color: #ffffff">
<table cellpadding=5 bgcolor="#cccccc" align="center">
<tr><td style="background-color: #cccccc">ユーザID</td><td>$id</td></tr>
<tr><td style="background-color: #cccccc">旧パスワード</td><td><input type="password" name="oldpass" size=20></td></tr>
<tr><td style="background-color: #cccccc">新パスワード</td><td><input type="password" name="pass" size=20></td></tr>
<tr><td style="background-color: #cccccc">新パスワード(確認)</td><td><input type="password" name="pass_conf" size=20></td></tr>
</table>
</td></tr>
</table>
<br>
<input type="submit" value="変更">
<input type="button" value="キャンセル" onClick="history.back()">
</div>
<p>
<hr>
<div align="center">$psmt_copyright</div>
</body>
</html>
EOF
    exit;
}

sub password_change_form {
    my($id) = @_;
    print <<EOF;
<input type="hidden" name="id" value="$id">
<div align="center">
<h2>PSMT パスワード</h2>
</div>
<hr>
<div align="center">
<table bgcolor="#ffffff" cellpadding=20 width="400" height="100">
<tr><td style="background-color: #ffffff">
<table cellpadding=5 bgcolor="#cccccc" align="center">
<tr><td style="background-color: #cccccc">ユーザID</td><td>$id</td></tr>
<tr><td style="background-color: #cccccc">現在のパスワード</td><td><input type="password" name="oldpass" size=20></td></tr>
<tr><td style="background-color: #cccccc">新パスワード</td><td><input type="password" name="pass" size=20></td></tr>
<tr><td style="background-color: #cccccc">新パスワード(確認)</td><td><input type="password" name="pass_conf" size=20></td></tr>
</table>
</td></tr>
</table>
<br>
<input type="submit" value="変更" onClick="SubmitAction(this.form,'pass_chg')">
<input type="button" value="キャンセル" onClick="history.back()">
</div>
<p>
<hr>
<div align="center">$psmt_copyright</div>
EOF
    exit;
}

sub user_list_form {
    
    print <<EOF;
<input type="hidden" name="uid" value="">
<input type="hidden" name="submode" value="">
<table width="100%">
<tr>
<td><span class="title">ProScan管理ツール</span>＞ユーザ管理</td><td align="right">ver. $psmt_version</td>
</tr>
</table>
<HR>
<table cellspasing=0 cellpadding=0 align="center">
<tr>
<td>
  【ユーザ一覧】<br>
  <table cellspacing=1 cellpadding=0 bgcolor=#000000>
  <tr><td>
  <table cellspacing=1 cellpadding=4 bgcolor=#000000>
  <tr bgcolor="#cccccc">
  <td>User ID</td><td>Password Expire date</td><td><br></td>
  </tr>
EOF
    
    tie( my %user_list, 'AnyDBM_File', $user_db_file, O_RDONLY, 0644 );
    foreach my $uid (sort keys %user_list) {
        my($pass,$expire_days,$expire) = split(/,/,$user_list{$uid});
	my $expire_str;
	if ( $expire_days != 0 ) {
	    my @T = localtime($expire);
	    $expire_str = sprintf("%04d/%02d/%02d",$T[5]+1900,$T[4]+1,$T[3]);
	} else {
	    $expire_str = "無期限";
	}
        print qq(  <tr bgcolor=#ffffff><th>$uid</th><th>$expire_str</th><td><input type="button" value="修正" onClick="SubmitUserForm(this.form,'user_edit','$uid')">);
	if ( $uid ne "admin" ) {
	    print qq(<input type="button" value="削除" onClick="UserDelConfirm(this.form,'$uid')">);
	}
	print qq(</td></tr>\n);
    }
    untie(%user_list);
    print <<EOF;
  </table>
  </td></tr>
  </table>
</td>
</tr>
<tr><td align="center"><br>
<input type="button" value="新規作成" onClick="SubmitUserForm(this.form,'user_new','')">
<input type="button" value="戻る" onClick="location.href='/cgi-bin/manager.cgi'">
</td></tr>
</table>
<hr>
<div align=right>$psmt_copyright</div>
EOF
}

sub user_entry_form {
    my $submode = $query->param('submode');
    my $uid = $query->param('uid');
    my $subtitle;
    my $uid_field;
    my $pass;
    my $expire;
    my $expire_days;
    my $expire_str;
    my $origin_pass;
    my @T;

    tie( my %user_list, 'AnyDBM_File', $user_db_file, O_RDONLY, 0644 );
    if ( $submode eq 'add' ) {
        $subtitle = "新規ユーザ作成";
        $uid = "";
        $uid_field = qq(<input type="text" name="uid" size="30">);
        $expire_days = 0;
        $expire_str = "無期限";
        $origin_pass = "";
    } else {
        $subtitle = "ユーザ情報変更";
        $uid_field = qq($uid);
        ($pass,$expire_days,$expire) = split(/,/,$user_list{$uid});
	if ( $expire_days != 0 ) {
	    @T = localtime($expire);
	    $expire_str = sprintf("次回更新日：%04d/%02d/%02d",$T[5]+1900,$T[4]+1,$T[3]);
	} else {
	    $expire_days = 0;
	    $expire_str = "無期限";
	}
	$origin_pass = qq(\n  <tr><td bgcolor="#cccccc">現在のパスワード</td><td bgcolor="#ffffff"><input type="password" name="oldpass" size=20></td></tr>);
        print qq(<input type="hidden" name="uid" value="">\n);
    }
    
    print <<EOF;
<input type="hidden" name="submode" value="">
<table width="100%">
<tr>
<td><span class="title">ProScan管理ツール</span>＞ユーザ管理＞$subtitle</td><td align="right">ver. $psmt_version</td>
</tr>
</table>
<HR>
<table cellspasing=0 cellpadding=0 align="center">
<tr>
<td>
  <table cellspacing=1 cellpadding=0 bgcolor=#000000>
  <tr><td>
  <table cellspacing=1 cellpadding=4 bgcolor=#000000>
  <tr><td bgcolor="#cccccc">ユーザID</td><td bgcolor="#ffffff">$uid_field</td></tr>$origin_pass
  <tr><td bgcolor="#cccccc">新パスワード</td><td bgcolor="#ffffff"><input type="password" name="pass" size=20></td></tr>
  <tr><td bgcolor="#cccccc">新パスワード（確認）</td><td bgcolor="#ffffff"><input type="password" name="pass_conf" size=20></td></tr>
  <tr><td bgcolor="#cccccc">パスワード有効日数</td><td bgcolor="#ffffff"><input type="text" name="expire_days" value="$expire_days" size=4>日<br>（$expire_str）</td></tr>
  </table>
  </td></tr>
  </table>
</td>
</tr>
<tr><td align="center"><br>
<input type="button" value="更新" onClick="SubmitUserAction(this.form,'$submode','$uid')">
<input type="button" value="戻る" onClick="history.back()">
</td></tr>
</table>
<hr>
<div align=right>$psmt_copyright</div>
EOF
}

sub GenPasswd {
    my($pass) = @_;
    my(@char) = ("a".."z","A".."Z","1".."8");
    my $t = time;
    my($a,$b) = $t =~ /(.)(.)$/;
    my $s = $char[$a] . $char[$b];
    return crypt($pass,$s);
}
sub user_action {
    my $session = &GetSessionID;
    my $submode = $query->param('submode');
    my $uid = $query->param('uid');
    my $oldpass = $query->param('oldpass');
    my $pass = $query->param('pass');
    my $pass_conf = $query->param('pass_conf');
    my $expire_days = $query->param('expire_days');
    my $crypt_pass;
    my $expire;
    my $oexpd;


    if ( $submode eq 'add' ) {
        &error("入力エラー","ユーザIDが未設定です。ユーザIDは必須です。") unless $uid;
        &error("入力エラー","パスワードが未設定です。パスワードは必須です。") unless $pass;
        &error("入力エラー","パスワードが一致しません。確認してください。") if $pass ne $pass_conf;
        #&error("入力エラー","有効期限が未設定です。有効期限は必須です。") unless $expire_days;
        $crypt_pass = &GenPasswd($pass);
        $expire = time + $expire_days * 86400;
	$expire = 0 unless $expire_days;
	tie( my %user_list, 'AnyDBM_File', $user_db_file, O_CREAT|O_RDWR, 0644 );
        $user_list{$uid} = "$crypt_pass,$expire_days,$expire";
	untie(%user_list);
    } elsif ( $submode eq 'del' ) {
        if($uid && $uid ne 'admin') {
  	    tie( my %user_list, 'AnyDBM_File', $user_db_file, O_CREAT|O_RDWR, 0644 );
            delete $user_list{$uid};
	    untie(%user_list);
        }
    } elsif ( $submode eq 'mod' ) {
        if( &CheckPassword($uid,$oldpass,1) ) {
            &error("パスワードエラー","現在のパスワードが正しくありません。");
        }
        &error("入力エラー","パスワードが一致しません。確認してください。") if $pass ne $pass_conf;
        #&error("入力エラー","有効期限が未設定です。有効期限は必須です。") unless $expire_days;
	tie( my %user_list, 'AnyDBM_File', $user_db_file, O_CREAT|O_RDWR, 0644 );
        if ( $pass ) {
            $crypt_pass = &GenPasswd($pass);
            $expire = time + $expire_days * 86400;
	    $expire = 0 unless $expire_days;
        } else {
            ($crypt_pass,$oexpd,$expire) = split(/,/,$user_list{$uid});
	    if ( $oexpd != $expire_days ) {
		if ( $expire_days != 0 ) {
		    $expire = time + $expire_days * 86400;
		} else {
		    $expire = 0;
		}
	    }
	    $pass = $oldpass;
        }
        $user_list{$uid} = "$crypt_pass,$expire_days,$expire";
	untie(%user_list);
        if ( $uid eq 'admin' ) {
            &regist_new_passwd($session,$uid,$pass);
        }
    }
    
    $mode = "user_f";
    &header($session);
    &user_list_form;
    #tie( %user_list, 'AnyDBM_File', $user_db_file, O_RDONLY, 0644 );
    #foreach (sort keys %user_list) {
    #    print "$_ => $user_list{$_}<br>\n";
    #}
}

sub option_form {
    # Group
    my $group = $query->param('group');
    my $group_name;
    my $group_select = "";
    my $def_group_select = "";
    my $undef_group_select = "";

    tie( my %group_data, 'AnyDBM_File', $group_db_file, O_RDONLY, 0644 );
    foreach (sort keys %group_data) {
        $group_name = $group_data{$_};
        my $selected = "";
        $selected = " selected" if $group eq $group_name;
        $group_select .= qq(<option value="$_"$selected>$group_name\n);
    }
    untie(%group_data);
    
    tie( my %client_data, 'AnyDBM_File', $data_file, O_RDONLY, 0644 );
    foreach (sort keys %client_data) {
        my @data = split(/,/,$client_data{$_});
        if($group && $data[1] eq $group) {
            $def_group_select .= qq(<option value="$data[0]">$_($data[0])\n);
        } elsif ($data[1]) {
            next;
        } else {
            $undef_group_select .= qq(<option value="$data[0]">$_($data[0])\n);
        }
    }
    untie(%client_data);
    
    my $acl_list;
    if(-f $acl_file) {
        open(ACL,$acl_file);
        my @acl = <ACL>;
        close(ACL);
        $acl_list = join("",@acl);
    }
    
    print <<EOF;
<input type="hidden" name="submode" value="">
<input type="hidden" name="def_list" value="">
<input type="hidden" name="undef_list" value="">
<table width="100%">
<tr>
<td><span class="title">ProScan管理ツール</span>＞オプション</td><td align="right">ver. $psmt_version</td>
</tr>
</table>
<HR>
<table cellspasing=0 cellpadding=0 align="center">
<tr>
<td>

<table align=center>
<tr><td valign="top" class="subtitle">管理パスワード設定</td>
<td>
<table cellpadding=5 bgcolor=#cccccc width=100%>
<tr><td><input type="text" size="20" name="manager_pass"></td></tr>
</table>
</td></tr>
<tr><td valign="top" class="subtitle">グループ設定</td>
<td>
<table cellpadding=5 bgcolor=#cccccc width=100%>
<tr><td nowrap>未設定管理対象サーバ一覧<br></td><td><br></td><td width="100%">グループ設定<br></td></tr>
<tr>

<td valign="top" align="right"><select size=17 name="undef_group" style="width: 250px;">
$undef_group_select</select>
</td>

<td valign="middle" align="center">
<input type="button" value="　→　" onClick="MoveDefSide(this.form)"><br>
<input type="button" value="　←　" onClick="MoveUndefSide(this.form)"><br></td>

<td valign="top"><select name="group" onChange="ChangeGroup(this.form)">
<option value="">（指定してください）</option>
$group_select</select>
<input type="button" value="削除" onClick="SubmitGroupAction(this.form,'del')"><br>
<input type="text" name="new_group_name"><input type="button" value="新規作成" onClick="SubmitGroupAction(this.form,'add')"><br>
グループ所属サーバ：<br>
<select size=13 name="def_group" style="width: 250px;">
$def_group_select</select>
</td>
</tr>
</table>
</td>
</tr>
<tr><td valign="top" class="subtitle">ACL設定</td>
<td>
<table cellpadding=5 bgcolor=#cccccc width=100%>
<tr><td nowrap>許可コンソール一覧<br></td></tr>
<tr>

<td valign="top"><textarea rows=10 cols=40 name="acl_list">
$acl_list</textarea>
</td>

</tr>
</table>
</td>
</tr>
</table>

</td>
</tr>
<tr><td align="center"><br>
<input type="button" value="反映" onClick="SubmitOptionAction(this.form)">
<input type="button" value="戻る" onClick="location.href='/cgi-bin/manager.cgi'">
</td></tr>
</table>
<hr>
<div align=right>$psmt_copyright</div>
EOF
}

sub group_action {
    my $submode = $query->param('submode');
    my $new_group = $query->param('new_group_name');
    my $group = $query->param('group');

    tie( my %group_list, 'AnyDBM_File', $group_db_file, O_CREAT|O_RDWR, 0644 );

    if ( $submode eq 'add' ) {
        &error("入力エラー","グループ名が未設定です。") unless $new_group;
        $group_list{$new_group} = "$new_group";
    } elsif ( $submode eq 'del' ) {
        &error("入力エラー","グループ名が未設定です。") unless $group;
        delete $group_list{$group};
        tie( my %client_data, 'AnyDBM_File', $data_file, O_RDWR, 0644 );
        foreach (sort keys %client_data) {
            my @data = csv2val($client_data{$_});
            if($group && $data[1] eq $group) {
                $client_data{$_} =~ s/,$group,/,,/;
            }
        }
        untie(%client_data);
    } elsif ( $submode eq 'mod' ) {
    }
    
    untie(%group_list);
    &header("");
    &option_form;
}

sub option_action {
    my $manager_pass = $query->param('manager_pass');
    my $group = $query->param('group');
    my $undef_list = $query->param('undef_list');
    my $def_list = $query->param('def_list');
    my $acl_list = $query->param('acl_list');
    
    &header("");

    tie( my %group_list, 'AnyDBM_File', $group_db_file, O_RDWR, 0644 );
    
    if ( $manager_pass ) {
        my $opt = '-bcd';
        $opt = '-bd' if -f $passwd_file;
        system("cd /tmp;/usr/bin/htpasswd $opt $passwd_file download $manager_pass");
        $opt = '-bd' if -f $passwd_file;
        system("cd /tmp;/usr/bin/htpasswd $opt $passwd_file admin $manager_pass");
        system("$command_dir/chg_passwd.pl $manager_pass");
    }
    
    if ( $acl_list ) {
        rename("$acl_file","$acl_file.bak");
        $acl_list =~ s/\x0D//g;
        $acl_list =~ s/\x0A+/\x0A/g;
        open(ACL,">$acl_file");
        print ACL "$acl_list\n";
        close(ACL);
    }
    
    if ( $undef_list ) {
        my @list = split(/,/,$undef_list);
        print "<!-- undef: @list -->\n";
        tie( my %client_data, 'AnyDBM_File', $data_file, O_RDWR, 0644 );
        foreach (sort keys %client_data) {
            my @data = split(/,/,$client_data{$_});
            if(grep($_ eq $data[0],@list)) {
                $data[1] = "";
                $client_data{$_} = join(",",@data);
            }
        }
        untie(%client_data);
    }
    
    if ( $def_list ) {
        my @list = split(/,/,$def_list);
        print "<!-- def: @list -->\n";
        tie( my %client_data, 'AnyDBM_File', $data_file, O_RDWR, 0644 );
        foreach (sort keys %client_data) {
            my @data = split(/,/,$client_data{$_});
            if(grep($_ eq $data[0],@list)) {
                $data[1] = "$group";
                $client_data{$_} = join(",",@data);
            }
        }
        untie(%client_data);
    }
    
    untie(%group_list);
    &option_form;
}

sub apply_list_form {
    my $update_list = "";
    my $scan_list = "";
    
    open(UPLIST,"$update_list_file");
    while(<UPLIST>) {
        s/\x0D//;
        $update_list .= $_;
    }
    close(UPLIST);
    
    open(SCANLIST,"$scan_list_file");
    while(<SCANLIST>) {
        s/\x0D//;
        $scan_list .= $_;
    }
    close(SCANLIST);
    
    print <<EOF;
<table width="100%">
<tr>
<td><span class="title">ProScan管理ツール</span>＞一括順序管理</td><td align="right">ver. $psmt_version</td>
</tr>
</table>
<HR>
<table cellspasing=0 cellpadding=0 align="center">
<tr>
<td>

<table cellspacing=5 cellpadding=2 align="center" bgcolor="#cccccc">
<tr>
<td>一括アップデート順序<br></td>
<td><br></td>
<td>一括スキャン順序</td>
</tr>
<tr>
<td><textarea name="update_list" rows=30 cols=40>$update_list</textarea></td>
<td><br></td>
<td><textarea name="scan_list" rows=30 cols=40>$scan_list</textarea></td>
</tr>
</table>
</td></tr>
<tr><td align="center">
<input type="button" value="反映" onClick="SubmitAction(this.form,'apply_act')">
<input type="button" value="戻る" onClick="location.href='/cgi-bin/manager.cgi'">
</td></tr>
</table>
<hr>
<div align=right>$psmt_copyright</div>
EOF
}

sub apply_action {
    my $update_list = $query->param('update_list');
    my $scan_list = $query->param('scan_list');
    
    &header("");
    
    rename("$update_list_file","$update_list_file.bak");
    open(UPLIST,">$update_list_file");
    $update_list =~ s/\x0D//;
    print UPLIST $update_list;
    close(UPLIST);
    
    rename("$scan_list_file","$scan_list_file.bak");
    open(SCANLIST,">$scan_list_file");
    $scan_list =~ s/\x0D//;
    print SCANLIST $scan_list;
    close(SCANLIST);
    
    $mode = "apply_f";
    &apply_list_form;
}

sub version_check {
    system("$command_dir/version_check.pl &");
    print <<EOF;
<table width="100%">
<tr>
<td><span class="title">ProScan管理ツール</span>＞最新情報を取得</td><td align="right">ver. $psmt_version</td>
</tr>
</table>
<HR>
<table cellspasing=10 cellpadding=0 align="center">
<tr><td>
現在、登録されているすべてのクライアント情報を取得しています。<br>
処理には時間がかかりますので、随時、ログを確認してください。<br>
</td></tr>
<tr><td align="center">
<input type="button" value="ログ確認" onClick="SubmitAction(this.form,'verchk_log')">
</td></tr>
<tr><td align="center">
<input type="button" value="戻る" onClick="location.href='/cgi-bin/manager.cgi'">
</td></tr>
</table>
<hr>
<div align=right>$psmt_copyright</div>
EOF
}

sub version_check_log {
    my $log_file = "$data_dir/version_check.log";
    open(LOG,"<:encoding(euc-jp)",$log_file);
    my @log = <LOG>;
    close(LOG);
    my $verchk_log = join(/\n/,@log);
    print <<EOF;
<table width="100%">
<tr>
<td><span class="title">ProScan管理ツール</span>＞最新情報を取得＞ログ確認</td><td align="right">ver. $psmt_version</td>
</tr>
</table>
<HR>
<table cellspasing=10 cellpadding=0 align="center">
<tr><td>
$verchk_log
</td></tr>
<tr><td align="center">
<input type="button" value="ログ確認" onClick="SubmitAction(this.form,'verchk_log')">
</td></tr>
<tr><td align="center">
<input type="button" value="戻る" onClick="location.href='/cgi-bin/manager.cgi'">
</td></tr>
</table>
<hr>
<div align=right>$psmt_copyright</div>
EOF
}

sub mk_config_file {
    my($config,$data) = @_;
    my @D = split(/,/,$data);
    my ($config_name,$config_filename,$max_scan_time,$max_recursion,$max_size,$max_ratio,$read_timeout,$exclude_mask,$include_mask,$repair_file,$match_action,$report_filename,$report_level,$output_filename,$show_level,$update_only,$cache_check) = @D;
    open(CONFIG,">$config_dir/$config.conf");
    binmode CONFIG, ":encoding(euc-jp)";
    print CONFIG <<"EOF";
\# auto create for PSMT : $config_name
[path]
DomainList=/etc/opt/proscan/domains
LicensePath=/var/opt/proscan/db/keys
LocalSocketPath=/var/opt/proscan/run
TempPath=/var/opt/proscan/tmp
UserFile=/var/opt/proscan/db/.users.db
DarkGrayList=/var/opt/proscan/db/darkgray.lst
LightGrayList=/var/opt/proscan/db/lightgray.lst
WgetPath=/usr/sbin/wget
S25RWhiteList=
CheckStartTimeFile=/var/run/filescan.start.time

[locale]
ErrorMessage=スキャン時にエラーが発生しました。
FilteredMessage=フィルタリング条件に一致しています。
InfectedMessage=ウイルスに感染しています。
OtherMessage=その他のエラーが発生しました。
ProtectedMessage=パスワード保護されています。
SuspiciousMessage=ウイルスに感染している疑いがあります。
SpamHighMessage=SPAMです。
SpamMediumMessage=SPAMの疑いがあります。
SpamLowMessage=SPAMの可能性があります。
TimeFormat=%H:%M:%S
DateFormat=%Y/%m/%d

[aveserver]
ProxyMode=no
ProxyScanners=10
ExecUser=root
ReportFileName=/var/opt/proscan/log/clamd.log
ReportLevel=7

[updater.options]
KeepSilent=yes
ReloadApplication=no
ShowExternalCmdOutput=no
UpdateHost=clamav-latest.promark-inc.com,clamav-latest-2.promark-inc.com
UpdatePort=80
UpdateProtocol=http
S25RWhiteListUrl=none

[scanner.options]
ArchiveScan=yes
MailboxScan=yes
MaxRatio=$max_ratio
MaxRecursion=$max_recursion
MaxScanTime=$max_scan_time
MaxSize=$max_size
ReadTimeout=$read_timeout
Recursion=yes
RepairFile=$repair_file
SaveDirectory=/var/opt/proscan/save
SendMail=no
Symlink=yes
UpdateOnly=$update_only
CacheCheck=$cache_check

[scanner.object]
IncludeMask=$include_mask
ExcludeMask=$exclude_mask
MatchAction=$match_action
ScanLevel=7

[scanner.display]
OutputFileName=$output_filename
ShowLevel=$show_level

[scanner.report]
ReportFileName=$report_filename
ReportLevel=$report_level

_group default

_end_group
EOF
    close(CONFIG);
    chmod(0644,"$config_dir/$config.conf");
}

sub config_form {
    my($config) = @_;
    my @config_list;
    my @config_name_list;
    my ($config_name,$config_filename,$max_scan_time,$max_recursion,$max_size,$max_ratio,$read_timeout,$exclude_mask,$include_mask,$repair_file,$match_action,$report_filename,$report_level,$output_filename,$show_level,$update_only,$cache_check);
    my $next_config;

    # config 情報取得
    tie( my %config_data, 'AnyDBM_File', $config_db_file, O_RDWR|O_CREAT, 0644 );
    foreach my $c (sort keys %config_data) {
        my $val = &decode_utf8($config_data{$c});
        my @D = split(/,/,$val);
        push(@config_list,$c);
        push(@config_name_list,$D[0]);
        if ( $c eq $config ) {
            ($config_name,$config_filename,$max_scan_time,$max_recursion,$max_size,$max_ratio,$read_timeout,$exclude_mask,$include_mask,$repair_file,$match_action,$report_filename,$report_level,$output_filename,$show_level,$update_only,$cache_check) = @D;
        }
        $next_config = ++$c;
    }
    
    # default
    unless ( @config_list ) {
        $config = "config0";
 	$next_config = "config1";
        $config_name_list[0] = "default";
        $config_name = "default";
        $config_filename = "scan.conf";
        $max_scan_time = 1200;
        $max_recursion = 5;
        $max_size = 0;
        $max_ratio = 200;
        $read_timeout = 120;
        $exclude_mask = "";
        $include_mask = "";
        $repair_file = "yes";
        $match_action = "delete";
        $report_filename = "/var/opt/proscan/log/filescanner.log";
        $report_level = 7;
        $output_filename = "";
        $show_level = 15;
	$update_only = "no";
	$cache_check = "no";
        $config_data{$config} = &encode_utf8("$config_name,$config_filename,$max_scan_time,$max_recursion,$max_size,$max_ratio,$read_timeout,$exclude_mask,$include_mask,$repair_file,$match_action,$report_filename,$report_level,$output_filename,$show_level,$update_only,$cache_check");
        mk_config_file($config,$config_data{$config});
    }
    untie( %config_data );
    
    my $mact_del;
    my $mact_move;
    my $mact_none;

    # form value set
    if ( $match_action eq "delete" ) {
        $mact_del = "selected";
    } elsif ( $match_action eq "move" ) {
        $mact_move = "selected";
    } else {
        $mact_none = "selected";
    }
    my $rep_level_1 = "checked" if $report_level == 7 || $report_level == 31;
    my $rep_level_2 = "checked" if $report_level == 31;
    my $out_level_1 = "checked" if $show_level & 1;
    my $out_level_2 = "checked" if $show_level & 2;
    my $out_level_3 = "checked" if $show_level & 4;
    my $out_level_4 = "checked" if $show_level & 8;
    my $out_level_5 = "checked" if $show_level & 256;

    my $update_only_yes = "checked" if $update_only eq "yes";
    my $cache_check_yes = "checked" if $cache_check eq "yes";
    
    print <<EOF;
<input type="hidden" name="submode" value="">
<input type="hidden" name="config" value="$config">
<script language="javascript">

function NewConfig(F,CONF) {
    F.config.value = CONF;
    F.config_name.value = "";
    F.scantime.value = "1200";
    F.recursion.value = "5";
    F.file_size.value = "0";
    F.ratio.value = "200";
    F.readtimeout.value = "120";
    F.exclude.value = "";
    F.include.value = "";
    F.match_action.selectedIndex = 0;
    F.report.value = "/var/opt/proscan/log/filescanner.log";
    F.report_level[0].checked = true;
    F.report_level[1].checked = false;
    F.output.value = "";
    F.show_level[0].checked = true;
    F.show_level[1].checked = true;
    F.show_level[2].checked = true;
    F.show_level[3].checked = false;
    F.update_only.checked = false;
    F.chache_check.checked = false;
    F.config_sel.selectedIndex = -1;
}

function EditConfig(F) {
    idx = F.config_sel.selectedIndex;
    if(idx < 0) {
        window.alert("コンフィグを選択して下さい。");
        return false;
    }
    F.config.value = F.config_sel.options[idx].value;
    F.mode.value = "config_f";
    F.submit();
}

function DeleteConfig(F) {
    idx = F.config_sel.selectedIndex;
    if(idx < 0) {
        window.alert("コンフィグを選択して下さい。");
        return false;
    }
    F.config.value = F.config_sel.options[idx].value;
    F.mode.value = "config_act";
    F.submode.value = "del";
    F.submit();
}

function CheckForm(F) {
    if ( F.config.value == "" ) {
        window.alert("新規作成またはコンフィグを選んで編集ボタンをクリックしてください。");
        return false;
    }
    if ( F.config_name.value == "" ) {
        window.alert("コンフィグの名前を設定してください。");
        return false;
    }
    F.mode.value = "config_act";
    F.submode.value = "add";
    F.submit();
}
</script>
<table width="100%">
<tr>
<td><span class="title">ProScan管理ツール</span>＞コンフィグ編集</td><td align="right">ver. $psmt_version</td>
</tr>
</table>
<HR>
<table cellspasing=0 cellpadding=0 align="center">
<tr>
<td>

<table cellspacing=10 cellpadding=0>
<tr><td>コンフィグ一覧<br></td><td><br></td><td width="100%">コンフィグ<br></td></tr>
<tr>
<td valign="top">
<select size=10 name="config_sel" style="width: 200px;">
EOF
    foreach my $val (@config_name_list) {
        my $c = shift(@config_list);
	my $sel = ($c eq $config) ? " selected":"";
        print qq(<option value="$c"$sel>$val</option>\n);
    }
    print <<EOF;
</select>
</td>
<td valign="top" align="center">
  <input type="button" value="新規作成" onClick="NewConfig(this.form,'$next_config')"><br>
  <input type="button" value="編集" onClick="return EditConfig(this.form)"><br>
  <input type="button" value="削除" onClick="return DeleteConfig(this.form)">
</td>
<td valign="top">
<table bgcolor="#cccccc">
<tr><td>名前</td><td><input type="text" name="config_name" size=30 value="$config_name"></td></tr>
<tr><td colspan=2>【スキャンオプション】</td></tr>
<tr><td>スキャンタイムアウト値</td><td><input type="text" size=10 name="scantime" value="$max_scan_time"></td></tr>
<tr><td>最大許容多重圧縮度</td><td><input type="text" size=10 name="recursion" value="$max_recursion"></td></tr>
<tr><td>最大許容アーカイブサイズ</td><td><input type="text" size=10 name="file_size" value="$max_size"></td></tr>
<tr><td>最大許容圧縮率</td><td><input type="text" size=10 name="ratio" value="$max_ratio"></td></tr>
<tr><td>ソケット読み込みタイムアウト値</td><td><input type="text" size=10 name="readtimeout" value="$read_timeout"></td></tr>
<tr><td>除外ファイルパターン</td><td><input type="text" size=60 name="exclude" value="$exclude_mask"></td></tr>
<tr><td>対象ファイルパターン</td><td><input type="text" size=60 name="include" value="$include_mask"></td></tr>
<tr><td>アクション</td><td><select name="match_action"><option value="none" $mact_none>何もしない</option><option value="move" $mact_move>移動</option><option value="delete" $mact_del>削除</option></select></td></tr>
<tr><td>ログファイル</td><td><input type="text" size=40 name="report" value="$report_filename"></td></tr>
<tr><td>ログ内容</td><td>
<input type="radio" name="report_level" value="7" $rep_level_1>通常
<input type="radio" name="report_level" value="31" $rep_level_2>詳細
</td></tr>
<tr><td>レポートファイル</td><td><input type="text" name="output" value="$output_filename" size=40></td></tr>
<tr><td>レポート内容</td><td>
<input type="checkbox" name="show_level" value="2" $out_level_2>バージョン、パラメータ
<input type="checkbox" name="show_level" value="4" $out_level_3>結果サマリ
<input type="checkbox" name="show_level" value="8" $out_level_4>チェック情報
<input type="checkbox" name="show_level" value="256" $out_level_5>詳細情報
</td></tr>
<tr><td>更新ファイルのみスキャン</td><td>
<input type="checkbox" name="update_only" value="yes" $update_only_yes>
</td></tr>
<tr><td>キャッシュチェック</td><td>
<input type="checkbox" name="cache_check" value="yes" $cache_check_yes>
</td></tr>
<tr><td><br></td><td></td></tr>
<tr><td></td><td><input type="button" value="更新" onClick="return CheckForm(this.form)"><input type="reset" value="キャンセル"></td></tr>
</table>
</td>
</tr>
</table>

</td>
</tr>
<tr><td align="center"><br>
<input type="button" value="戻る" onClick="location.href='/cgi-bin/manager.cgi'">
</td></tr>
</table>
<hr>
<div align=right>$psmt_copyright</div>
EOF
}

sub config_action {
    my $submode = $query->param('submode');
    my $config = $query->param('config');
    my $config_name = $query->param("config_name");
    my @D;
    my @val;

    tie( my %config_data, 'AnyDBM_File', $config_db_file, O_CREAT|O_RDWR, 0644 );
    unless ($config) {
        $mode = "config_f";
        &header("");
        &config_form("");
    }
    if ( $submode eq 'add' ) {
        $D[0] = $config_name;
        $D[1] = "scan.conf";
        $D[2] = $query->param("scantime");
        $D[3] = $query->param("recursion");
        $D[4] = $query->param("file_size");
        $D[5] = $query->param("ratio");
        $D[6] = $query->param("readtimeout");
        $D[7] = $query->param("exclude");
        $D[8] = $query->param("include");
        $D[9] = $query->param("repair");
        $D[10] = $query->param("match_action");
        $D[11] = $query->param("report");
        @val = $query->param("report_level");
        $D[12] = eval(join('+',@val));
        $D[13] = $query->param("output");
        @val = $query->param("show_level");
        $D[14] = eval(join("+",@val)) + 1;
	$D[15] = ( $query->param("update_only") eq "yes" ) ? "yes" : "no";
	$D[16] = ( $query->param("cache_check") eq "yes" ) ? "yes" : "no";
        $config_data{$config} = &encode_utf8(join(",",@D));
        mk_config_file($config,$config_data{$config});
    } elsif ( $submode eq 'del' ) {
        delete $config_data{$config};
	unlink("$config_dir/$config.conf");
        $config = "";
    }
    
    untie(%config_data);
    $mode = "config_f";
    &header("");
print "<!-- $config_name -->\n";
    &config_form($config);
}

sub activation_form {
    my($msg,$color) = @_;
    my $group_select_tag = qq(<option value="">（選択してください）</option>\n);
    my $config_select_tag = qq(<option value="">（選択してください）</option>\n);
    my @D;

    # group list
    tie( my %group_data, 'AnyDBM_File', $group_db_file, O_RDONLY, 0644 );
    foreach my $g (sort keys %group_data) {
        $group_select_tag .= qq(<option value="$g">$group_data{$g}</option>\n);
    }
    untie(%group_data);
    # config 情報取得
    tie( my %config_data, 'AnyDBM_File', $config_db_file, O_RDONLY, 0644 );
    foreach my $c (sort keys %config_data) {
        @D = split(/,/,&decode_utf8($config_data{$c}));
        $config_select_tag .= qq(<option value="$c">$D[0]</option>\n);
    }
    untie(%config_data);
    
    print <<EOF;
<script language="javascript">
function Activate(F) {
    document.getElementById('msg').innerHTML = "";
    if ( F.agent.value == "" || F.passwd.value == "" ) {
        window.alert("「対象クライアント」および「パスワード」は必須です");
        return false;
    }
    F.mode.value = "active";
    F.submit();
}
</script>
<table width="100%">
<tr>
<td><span class="title">ProScan管理ツール</span>＞アクティベーション</td><td align="right">ver. $psmt_version</td>
</tr>
</table>
<HR>
<table cellspasing=0 cellpadding=0 align="center">
<tr>
<td>

<table cellspacing=5 cellpadding=2 align="center" bgcolor="#cccccc">
<tr>
<td align="right">対象クライアント<br></td>
<td><input type="text" name="agent" size=20>※IPアドレスまたはホスト名で指定</td>
</tr>
<tr>
<td align="right">パスワード<br></td>
<td><input type="password" name="passwd" size=20>※管理マネージャのパスワードを設定</td>
</tr>
<tr>
<td align="right">グループ<br></td>
<td valign="top">
<select name="group">
$group_select_tag
</select>
</td>
</tr>
<tr>
<td align="right">セットアップ用設定ファイル<br></td>
<td valign="top">
<select name="config">
$config_select_tag
</select>
</td>
</tr>
</table>
<br>
<div id="msg" align="center" style="size: 120%;color: $color;">$msg</div>
<br>

</td>
</tr>
<tr><td align="center"><br>
<input type="button" value="アクティベーション" onClick="return Activate(this.form)">
<input type="button" value="戻る" onClick="location.href='/cgi-bin/manager.cgi'">
</td></tr>
</table>
<hr>
<div align=right>$psmt_copyright</div>
EOF
}

sub activation {
    my $agent = $query->param('agent');
    my $manager_pass = $query->param('passwd');
    my $group = $query->param('group');
    my $type = $query->param('type');
    my $config = $query->param('config');
    my $ip;
    my $data;
    my $ip_addr;

    if( $agent =~ /^\d+\.\d+\.\d+\.\d+$/ ) {
        $ip = $agent;
    } else {
        $ip_addr = inet_aton($agent);
        $ip = sprintf("%d.%d.%d.%d",unpack("CCCC",$ip_addr));
    }
    
    if ( -f $passwd_file ) {
        open(PASS,"$passwd_file");
        $data = <PASS>;
        chomp($data);
        close(PASS);
    } else {
        $data = "dummy:abc";
    }
    my ($user,$pass) = split(/:/,$data);
    my ($salt) = $pass =~ /^(..)/;
    my $crypt_pass = crypt($manager_pass,$salt);
    my $msg;
    my $color;

    if ( $pass ne $crypt_pass ) {
        $msg = "パスワードが正しくありません";
        $color = "red";
        &header;
        &activation_form($msg,$color);
        return;
    }
    $rc = system("$active_command $ip $manager_pass");
    
    if ( $rc != 0 ) {
        $msg = "アクティベーションに失敗しました<br>すでにアクティベーション済みか、エージェントが起動していません";
        $color = "red";
    } else {
        # activation complete
        tie( my %client_data, 'AnyDBM_File', $data_file, O_CREAT|O_RDWR, 0644 );
        my $addr = inet_aton($ip);
        my $host = gethostbyaddr($addr,AF_INET);
	$host = $ip unless $host;
        $client_data{$host} = "$ip,$group,$type,,,,,,,,,,,,,$config";
        #  - setup config
        my $config_file = "$config_dir/$config.conf";
        link($config_file,"$download_dir/scan.conf");
        my $rc1 = system("$setup_command $ip config");
        #  - setup group/type
        open(GRP,">$download_dir/group");
        print GRP "$group:$type";
        close(GRP);
        my $rc2 = system("$setup_command $ip group");
        # - setup license
        link($license_file_s390,"$download_dir/proscan_s390.key");
        #link($license_file_x86,"$download_dir/proscan_i386.key");
        my $rc3 = system("$setup_command $ip license");
        if( $rc1 != 0 || $rc2 != 0 || $rc3 != 0 ) {
            $msg = "セットアップに失敗しました($rc1,$rc2,$rc3)<br>個別にセットアップを行ってください";
            $color = "red";
	    #delete($client_data{$host});
        } else {
            $msg = "アクティベーションが完了しました";
            $color = "blue";
        }
        untie(%client_data);
    }
    &header;
    &activation_form($msg,$color);
}

sub license_form {
    my($msg,$color) = @_;

    # get license info
    my $s390_license_info = "ライセンスファイルがありません";
#    my $i386_license_info = "ライセンスファイルがありません";

    my @lic_type_str = ("Trial","Normal","Unlimited");

    if ( -f "$license_file_s390" ) {
	my $lic_s390 = `$show_license_command $license_file_s390`;
 	chomp($lic_s390);
	my ($reg_code,$type,$domain,$user,$expire) = split(/,/,$lic_s390);
	$s390_license_info = "Registration code : $reg_code<br>License type : $lic_type_str[$type]<br>Expire date : $expire";
    }
#    if ( -f "$license_file_x86" ) {
#	my $lic_i386 = `$show_license_command $license_file_x86`;
# 	chomp($lic_i386);
#	my ($reg_code,$type,$domain,$user,$expire) = split(/,/,$lic_i386);
#	$i386_license_info = "Registration code : $reg_code<br>License type : $lic_type_str[$type]<br>Expire date : $expire";
#    }

    print <<EOF;
<table width="100%">
<tr>
<td><span class="title">ProScan管理ツール</span>＞ライセンス管理</td><td align="right">ver. $psmt_version</td>
</tr>
</table>
<HR>
<table cellspasing=0 cellpadding=0 align="center">
<tr>
<td>
ライセンスファイルを指定してアップロードしてください。<br>
<table cellspacing=5 cellpadding=2 align="center" bgcolor="#cccccc">
<tr>
<td align="left" colspan="2">zSeries ライセンス(s390)</td>
</tr>
<tr>
<td align="right">現在のライセンス情報</td>
<td bgcolor="#eeeeee">$s390_license_info<br></td>
</tr>
<tr>
<td align="right">アップロードファイル</td>
<td><input type="file" name="s390" size=50 onClick="document.getElementById('msg').innerHTML=''"></td>
</tr>
<!--
<tr>
<td align="center" colspan="2"><hr></td>
</tr>
<tr>
<td align="left" colspan="2">xSeries ライセンス(i386)<br></td>
</tr>
<tr>
<td align="right">現在のライセンス情報</td>
<td bgcolor="#eeeeee">\$i386_license_info<br></td>
</tr>
<tr>
<td align="right">アップロードファイル</td>
<td><input type="file" name="i386" size=50 onClick="document.getElementById('msg').innerHTML=''"></td>
</tr>
-->
</table>
<br>
<div id="msg" align="center" style="size: 120%;color: $color;">$msg</div>
<br>

</td>
</tr>
<tr><td align="center"><br>
<input type="button" value="アップロード" onClick="return UploadCheck(this.form)">
<input type="button" value="戻る" onClick="location.href='/cgi-bin/manager.cgi'">
</td></tr>
</table>
<hr>
<div align=right>$psmt_copyright</div>
EOF
}

sub copy {
    my($src,$dest) = @_;
    
    if ( -f $dest ) {
        unlink($dest);
    }
    open(SRC,$src);
    open(DST,">$dest");
    while(<SRC>) {
        print DST;
    }
    close(DST);
    close(SRC);
}

sub get_file {
    my($file_handle,$save_file) = @_;
    my $buffer;
    my $file = "";
    my $size = 0;

    binmode($file_handle,":encoding(euc-jp)");
    while(my $n=read($file_handle, $buffer, 16384)){ 
        $file .= $buffer;
        $size += $n;
    } 
    
    # ファイルを指定ディレクトリにコピー 
    open OUT, ">:encoding(euc-jp)", $save_file ;
    print OUT $file;
    close(OUT); 
    
    # パーミッションを変更 
    chmod (0644, "$save_file"); 
    
    return $size;
}

sub license_upload {
    my $msg = "ライセンスファイルをアップロードしました";
    my $color = "blue";
    my $filename1 = $query->param('s390'); # ファイル名の取得 
    if ( $filename1 ) {
        my $size = &get_file($filename1,$license_file_s390);
        if ( $size != 854 && $size != 1633 ) {
            $msg = "ライセンスファイルではありません(2)";
            $color = "red";
        }
    }
#    my $filename2 = $query->param('i386'); # ファイル名の取得 
#    if ( $filename2 ) {
#        my $size = &get_file($filename2,$license_file_x86);
#        if ( $size != 854 && $size != 1633 ) {
#            $msg = "ライセンスファイルではありません(3)";
#            $color = "red";
#        }
#    }
    &header;
    &license_form($msg,$color);
}

sub control_form {
    my($msg,$color) = @_;
    
    $h = $query->param('client');
    
    # distribution status
    open(VER,"$dist");
    my $dist_version = <VER>;
    close(VER);
    chomp($dist_version);
    my ($proscan_dist,$products_dist,$engine_dist,$plib_dist,$format_dist,$vdf_dist,$signature_dist,$update_dist) = split(/,/,$dist_version);
    my @T = localtime($update_dist);
    my $dist_update_time = sprintf("%04d/%02d/%02d %02d:%02d:%02d",$T[5]+1900,$T[4]+1,$T[3],$T[2],$T[1],$T[0]);
    
    if ( $h ) {
        tie( my %client_data, 'AnyDBM_File', $data_file, O_RDONLY, 0644 );
        my @D = split(/,/,$client_data{$h});
        my $status = &status_check($dist_version,$client_data{$h});
        my $status_icon = "<font color=green>●</font>";
        $status_icon = "<font color=yellow>●</font>" if $status == 1;
        $status_icon = "<font color=red>●</font>" if $status == 2;
        
        my $result_icon = "<font color=green>●</font>";
        $result_icon = "<font color=yellow>●</font>" if $D[13] eq "ERR" || $D[13] eq "";
        $result_icon = "<font color=red>●</font>" if $D[13] >= 1;
        untie(%client_data);
        
        my $type_normal = ($D[2] eq 'normal') ? "checked" : "";
        my $type_test = ($D[2] eq 'test') ? "checked" : "";
        
        # config 情報取得
 	my @config_list=();
	my @config_name_list=();
        tie( my %config_data, 'AnyDBM_File', $config_db_file, O_RDONLY, 0644 );
        foreach my $c (sort keys %config_data) {
            my @C = split(/,/,$config_data{$c});
            push(@config_list,$c);
            push(@config_name_list,$C[0]);
        }
        untie(%config_data);
	
	# cronスケジュール情報
	my $schedule_timing = "スケジュールされていません";
	my $schedule_js = qq(onFocus="this.value='';this.onfocus=function(){true;}");
	if ( $D[16] ne "" ) {
	    $schedule_timing = $D[16];
	    $schedule_js = "";
	}
        
        print <<EOF;
<input type="hidden" name="submode" value="">
<input type="hidden" name="client" value="$h">
<input type="hidden" name="client_ip" value="$D[0]">
<input type="hidden" name="passwd" value="">
<input type="hidden" name="group" value="$D[1]">
<table width="100%">
<tr>
<td><span class="title">ProScan管理ツール</span>＞コントロール</td><td align="right">ver. $psmt_version</td>
</tr>
</table>
<HR>
<table cellspasing=0 cellpadding=0 align="center">
<tr>
<td>

<table cellspacing=5 cellpadding=2 align="center" bgcolor="#cccccc">
<tr>
<td>ホスト名<br></td>
<td bgcolor="white">$h<br></td>
</tr>
<tr>
<td>IPアドレス<br></td>
<td bgcolor="white">$D[0]<br></td>
</tr>
<tr>
<td>グループ<br></td>
<td bgcolor="white">$D[1]<br></td>
</tr>
<tr>
<td colspan="2"><hr></td>
</tr>
<tr>
<td>状態<br></td>
<td>バージョン：$status_icon　　スキャン：$result_icon　　<input type="button" value="スキャン結果確認" onClick="ScanLog('$D[0]')"></td>
</tr>
<tr>
<td colspan="2"><hr></td>
</tr>
<tr>
<td>【バージョン情報】<br></td>
<td><br></td>
</tr>
<tr>
<td>ProScanバージョン<br></td>
<td bgcolor="white">$D[4]<br></td>
</tr>
<tr>
<td>エンジンバージョン<br></td>
<td bgcolor="white">$D[6]<br></td>
</tr>
<tr>
<td>VDFバージョン<br></td>
<td bgcolor="white">$D[9]<br></td>
</tr>

<!-- オンデマンド実行ボタン　start  -->
<tr>
<td colspan="2"><hr></td>
</tr>
<tr>
<td>【オンデマンド】<br></td>
<td>最新情報<br></td>
</tr>
<tr>
<td><input type="button" value="VDFアップデート" onClick="ControlAction(this.form,'update')"></td>
<td bgcolor="white">$D[11]<br></td>
</tr>
<tr>
<td rowspan=2><input type="button" value="スキャン実施" onClick="ControlAction(this.form,'scan')"></td>
<td>スキャン対象：<input type="text" name="scan_target" value="" size=60><br></td>
</tr>
<tr>
<td bgcolor="white">$D[14]<br></td>
</tr>
<tr>
<td><input type="button" value="コンフィグ反映" onClick="ControlAction(this.form,'config')"></td>
<td>
<select name="config_sel">
EOF
        foreach my $val (@config_name_list) {
            my $c = shift(@config_list);
            my $sel = ($c eq $D[15]) ? " selected" : "";
            print qq(<option value="$c"$sel>$val</option>\n);
        }
        print <<EOF;
</select>
&nbsp;ファイル指定：<input type="file" name="config_file">
</td>
</tr>
<tr>
<td><input type="button" value="ライセンス更新" onClick="ControlAction(this.form,'license')"></td>
<td bgcolor="white">更新期限：$D[12]<br></td>
</tr>
<tr>
<td><input type="button" value="スケジュール変更" onClick="ControlAction(this.form,'schedule')"></td>
<td><input type="text" name="schedule" value="$schedule_timing" size=30 $schedule_js><span style="font-size:9pt;">※cronへのスケジュール設定スタイルで指定</span><br></td>
</tr>

<!-- 制御コマンド実行ボタン　start  -->
<tr>
<td colspan="2"><hr></td>
</tr>
<tr>
<td colspan="2" align="center"><input type="button" value="アップグレード" onClick="ControlAction(this.form,'upgrade')">
<input type="button" value="ディアクティベート" onClick="ControlAction(this.form,'deactivate')"></td>
</tr>
</table>
<br>
<div id="msg" align="center" style="size: 120%;color: $color;">$msg</div>
<br>

</td>
</tr>
<tr><td align="center"><br>
<input type="button" value="戻る" onClick="location.href='/cgi-bin/manager.cgi'">
</td></tr>
</table>
<hr>
<div align=right>$psmt_copyright</div>
EOF
    } else {
        # target 取得
        my $list_file = $query->param('target_list');
 	my $list = "";
        
        if ( -f $list_file ) {
            open(LIST,$list_file);
            while(<LIST>) {
                $list .= $_;
            }
            close(LIST);
        }
        
        # config 情報取得
 	my @config_list = ();
	my @config_name_list = ();
        tie( my %config_data, 'AnyDBM_File', $config_db_file, O_RDONLY, 0644 );
        foreach my $c (sort keys %config_data) {
            my @C = split(/,/,$config_data{$c});
            push(@config_list,$c);
            push(@config_name_list,$C[0]);
        }
        untie(%config_data);
        
        print <<EOF;
<input type="hidden" name="submode" value="">
<input type="hidden" name="target_list" value="$list_file">
<table width="100%">
<tr>
<td><span class="title">ProScan管理ツール</span>＞コントロール（リスト）</td><td align="right">ver. $psmt_version</td>
</tr>
</table>
<HR>
<table cellspasing=0 cellpadding=0 align="center">
<tr>
<td>

<table cellspacing=5 cellpadding=2 align="center" bgcolor="#cccccc">
<tr>
<td>【対象一覧】<br></td>
</tr>
<tr>
<td><textarea rows=20 cols=60>$list</textarea></td>
</tr>

<tr>
<td><input type="button" value="VDFアップデート" onClick="ControlAction(this.form,'update')"></td>
</tr>
<tr>
<td>スキャン対象：<input type="text" name="scan_target" value="" size=60><input type="button" value="スキャン実施" onClick="ControlAction(this.form,'scan')"></td>
</tr>

<tr>
<td>
<select name="config_sel">
EOF
        foreach my $val (@config_name_list) {
            my $c = shift(@config_list);
            print qq(<option value="$c">$val</option>\n);
        }
        print <<EOF;
</select><input type="button" value="コンフィグ反映" onClick="ControlAction(this.form,'config')"><br>
</td>
</tr>
<tr>
<td><input type="button" value="ライセンス更新" onClick="ControlAction(this.form,'license')"></td>
</tr>
<tr>
<td><input type="button" value="アップグレード" onClick="ControlAction(this.form,'upgrade')"></td>
</tr>
</table>
<br>
<div id="msg" align="center" style="size: 120%;color: $color;">$msg</div>
<br>

</td>
</tr>
<tr><td align="center"><br>
<input type="button" value="戻る" onClick="location.href='/cgi-bin/manager.cgi'">
</td></tr>
</table>
<hr>
<div align=right>$psmt_copyright</div>
EOF
    }
}

sub control_action {
    my $submode = $query->param('submode');
    my $client = $query->param('client');
    my $ip = $query->param('client_ip');
    my $msg;
    my $color;
    
    if ( $submode eq 'update' ) {
        ($msg,$color) = &control_update($ip);
    } elsif ( $submode eq 'scan' ) {
        ($msg,$color) = &control_scan($ip);
    } elsif ( $submode eq 'license' ) {
        ($msg,$color) = &control_license($ip);
    } elsif ( $submode eq 'upgrade' ) {
        ($msg,$color) = &control_upgrade($ip);
    } elsif ( $submode eq 'config' ) {
        ($msg,$color) = &control_config($ip,$client);
    } elsif ( $submode eq 'type_change' ) {
        my $group = $query->param('group');
        my $type = $query->param('type');
        ($msg,$color) = &control_type($ip,$group,$type,$client);
    } elsif ( $submode eq 'deactivate' ) {
        my $pass = $query->param('passwd');
        ($msg,$color) = &control_deact($ip,$pass,$client);
    } elsif ( $submode eq 'schedule' ) {
	my $timing = $query->param('schedule');
        ($msg,$color) = &control_schedule($ip,$client,$timing);
    } else {
        $msg = "処理できませんでした($submode,$ip)";
        $color = "red";
    }
    
    &header;
    &control_form($msg,$color);
}

sub control_schedule {
    my($ip,$client,$timing) = @_;
    my $result;
    my $msg;
    my $color;
    
    open(FILE,">$download_dir/schedule");
    print FILE "$timing\n";
    close(FILE);

    if ($ip) {
        $result = `echo setup schedule | $manager_command -a $ip`;
    } else {
        $result = &target_control('setup','schedule');
    }
    
    if ( $result =~ /^OK/ ) {
        $msg = "cronのスケジュール更新を完了しました";
        $color = "blue";
	tie( my %client_data, 'AnyDBM_File', $data_file, O_CREAT|O_RDWR, 0644 );
	my $data = $client_data{$client};
	my @D = split(/,/,$data);
	$D[16] = $timing;
	$client_data{$client} = join(",",@D);
	untie(%client_data);
    } else {
        $msg = "cronのスケジュール更新に失敗しました";
        $color = "red";
    }
    return ($msg,$color);
}

sub control_update {
    my($ip) = @_;
    my $update_url = $manager_ipaddr;
    my $result;
    my $msg;
    my $color;
    
    if ($ip) {
        $result = `echo update $update_url | $manager_command -a $ip`;
    } else {
        $result = &target_control('update',$update_url);
    }
    if ( $result =~ /^OK/ ) {
        $msg = "アップデートを完了しました";
        $color = "blue";
    } else {
        $msg = "アップデートに失敗しました";
        $color = "red";
    }
    return ($msg,$color);
}

sub control_scan {
    my($ip) = @_;
    my $target = $query->param('scan_target');
    my $result;
    my $msg;
    my $color;
    
    unless ( $target ) {
        return("スキャン対象を設定してください","red");
    }
    if ($ip) {
        $result = `echo scan $target | $manager_command -a $ip`;
    } else {
        $result = &target_control('scan',$target);
    }
    if ( $result =~ /^OK/ ) {
        $msg = "スキャンを完了しました";
        $color = "blue";
    } else {
        $msg = "スキャンに失敗しました";
        $color = "red";
    }
    return ($msg,$color);
}

sub control_license {
    my($ip) = @_;
    my $result;
    my $msg;
    my $color;
    
    &copy($license_file_s390,"$download_dir/proscan_s390.key");
    #&copy($license_file_x86,"$download_dir/proscan_i386.key");

    if ($ip) {
        $result = `echo setup license | $manager_command -a $ip`;
    } else {
        $result = &target_control('setup','license');
    }
    
    if ( $result =~ /^OK/ ) {
        $msg = "ライセンスの更新を完了しました";
        $color = "blue";
    } else {
        $msg = "ライセンスの更新に失敗しました";
        $color = "red";
    }
    return ($msg,$color);
}

sub control_config {
    my($ip,$client) = @_;
    my $result;
    my $msg;
    my $color;
    my $flag = 0;
    
    my $config = $query->param('config_sel');
    my $config_file;
    my $config_file_handle = $query->param('config_file');

    # config file取得
    if ( $config_file_handle ) {
	$config_file = "$config_dir/temp$$.conf";
	# file upload
        my $size = &get_file($config_file_handle,$config_file);
        $flag = 1;
    } else {
	# select config file
	$config_file = "$config_dir/$config.conf";
    }
    &copy($config_file,"$download_dir/scan.conf");

    if ($ip) {
        $result = `echo setup config | $manager_command -a $ip`;
    } else {
        $result = &target_control('setup','config');
    }
    
    if ( $result =~ /^OK/ ) {
        $msg = "コンフィグファイルの更新を完了しました";
        $color = "blue";
        tie( my %client_data, 'AnyDBM_File', $data_file, O_RDWR, 0644 );
        
        if ($ip) {
            my @D = split(/,/,$client_data{$client});
            $D[15] = $config;
            $client_data{$client} = join(",",@D);
        } else {
            open(HOST,$target_list_file);
            while(<HOST>) {
                chomp;
                my @D = split(/,/,$client_data{$_});
                $D[15] = $config;
                $client_data{$_} = join(",",@D);
            }
            close(HOST);
        }
        
        untie(%client_data);
    } else {
        $msg = "コンフィグファイルの更新に失敗しました";
        $color = "red";
    }

    if ( $flag == 1 ) {
	unlink($config_file);
    }

    return ($msg,$color);
}

sub control_upgrade {
    my($ip) = @_;
    my $result;
    my $msg;
    my $color;
    
    if ($ip) {
        $result = `echo upgrade | $manager_command -a $ip`;
    } else {
        $result = &target_control('upgrade','');
    }
    
    if ( $result =~ /^ERR/ ) {
        $msg = "アップグレードでエラーが発生しました";
        $color = "red";
    } else {
        $result = `echo ping | $manager_command -a $ip`;
        if ( $result =~ /^OK/ ) {
            $msg = "アップグレードを完了しました";
            $color = "blue";
	} else {
            $msg = "アップグレードに失敗しました";
            $color = "red";
	}
    }
    return ($msg,$color);
}

sub control_type {
    my($ip,$group,$type,$client) = @_;
    my $result;
    my $msg;
    my $color;
    
    #  - setup group/type
    open(GRP,">$download_dir/group");
    print GRP "$group:$type";
    close(GRP);
    
    if ($ip) {
        $result = `echo setup group | $manager_command -a $ip`;
    } else {
        $result = "error";
    }
    
    if ( $result =~ /^OK/ ) {
        tie( my %client_data, 'AnyDBM_File', $data_file, O_RDWR, 0644 );
        my @D = split(/,/,$client_data{$client});
        $D[2] = $type;
        $client_data{$client} = join(",",@D);
        untie(%client_data);
        $msg = "タイプ変更を完了しました";
        $color = "blue";
    } else {
        $msg = "タイプ変更に失敗しました";
        $color = "red";
    }
    return ($msg,$color);
}

sub control_deact {
    my($ip,$pass,$client) = @_;
    my $data;
    my $result;
    my $msg;
    my $color;
    
    if ( -f $passwd_file ) {
        open(PASS,"$passwd_file");
        $data = <PASS>;
        chomp($data);
        close(PASS);
    } else {
        $data = "dummy:abc";
    }
    my ($user,$mpass) = split(/:/,$data);
    my ($salt) = $mpass =~ /^(..)/;
    my $crypt_pass = crypt($pass,$salt);
    if ( $mpass ne $crypt_pass ) {
        $msg = "パスワードが正しくありません";
        $color = "red";
        return($msg,$color);
    }
    
    if ($ip) {
        $result = `echo deactivate  | $manager_command -a $ip`;
    } else {
        $result = "error";
    }
    
    if ( $result =~ /^OK/ ) {
        tie( my %client_data, 'AnyDBM_File', $data_file, O_RDWR, 0644 );
        delete $client_data{$client};
        untie(%client_data);
        $msg = "ディアクティベートを完了しました";
        $color = "blue";
    } else {
        $msg = "ディアクティベートに失敗しました";
        $color = "red";
    }
    return ($msg,$color);
}

sub target_control {
    my($command,$param) = @_;
    
    my $results = `$all_command $command "$param" -f $target_list_file`;
## for debug 2008.1.10
open(LOG,">>$manager_dir/target_control.log");
print LOG qq(Command: $all_command $command "$param" -f $target_list_file\n);
print LOG "Results:\n";
print LOG $results;
close(LOG);
##
    my ($err) = $results =~ / Failed\s+:\s(\d+)/;
    
    return "OK" if $err == 0;
    return "ERR";
}

#sub rollback_action {
#    my $page = $query->param('page');
#    my $level = $query->param('rollback');
#    
#    my $log = `$command_dir/all_rollback.pl $level`;
#    
#    $mode = "list";
#    
#    &header;
#    &list($page);
#    #print "<pre>$log</pre>\n";
#}

sub select_action {
    $mode = "list";
    $page = 1;
    
    if($host_sel || $group_sel || $ver_stat_sel || $proscan_sel || $engine_sel || $vdf_sel || $update_sel || $scan_stat_sel || $scan_date_sel) {
        $selecting_flag = "on";
    }
    
    &header;
    &list($page);
}

sub client_dump {
        tie( my %client_data, 'AnyDBM_File', $data_file, O_RDWR, 0644 );
        foreach (sort keys %client_data) {
            print "$_ : $client_data{$_}<br>\n";
        }
        untie(%client_data);

}

sub db_dump {
    my ($dbname) = @_;
    my $db_file = "${dbname}_db_file";
    print "<!-- DB DUMP START: $db_file -->\n";
    tie( my %data, 'AnyDBM_File', $db_file, O_RDONLY, 0644 );
    foreach (sort keys %data) {
        print "$_ : $data{$_}<br>\n";
    }
    untie(%data);
}
