#!/usr/bin/perl
#############################################################################
#
use strict;
use warnings;

use Getopt::Long;
#$Getopt::Long::ignorecase=undef;

my $opt_t = "/var/www/manager/update/latest";
my $opt_m = "http://clamav-latest.promark-inc.com";
my $opt_h = "";
my $opt_p = 3128;
my $opt_u = "";
my $opt_w = "";
GetOptions( "target=s" => \$opt_t,
	"proxy-host=s" => \$opt_h,
	"proxy-port=i" => \$opt_p,
	"proxy-user=s" => \$opt_u,
	"proxy-password=s" => \$opt_w );

my $clamdb = $opt_t;
my $ium = "/var/www/manager/mirror";
# mirror where files such as daily-12133.cdiff exist
my $mirror = $opt_m;
my $rc = 0;
my $WGET = `which wget`;
chomp $WGET;
my $proxy = "";

if ( $opt_h ne "" ) {
    $proxy = "http://$opt_h";
    if ( $opt_p ne "" ) {
	$proxy .= ":$opt_p";
    }
    #$WGET .= " -e 'http_proxy=$proxy'";
    if ( $opt_u ne "" ) {
	$WGET .= " --proxy-user=$opt_u --proxy-password=$opt_w";
    }
    $ENV{'http_proxy'} = $proxy;
    print "use proxy: $proxy\n";
}

# get the TXT record for clamav version
my $txt = "";
$txt = getTXT2("clamav-latest.promark-inc.com");
$txt = getTXT2("clamav-latest-2.promark-inc.com") unless $txt;

exit 1 unless $txt;

chdir($clamdb) || die ("Can't chdir to $clamdb : $!\n");

# dump the record in a file
print "TXT from DNS: $txt\n";
open D, ">dns.txt";
print D "$txt";
close D;

# temp dir for wget updates
mkdir("$clamdb/temp");

# get what we need
my ( $clamv, $mainv , $dailyv, $x, $y, $z, $safebrowsingv, $bytecodev ) = split /:/, $txt ;

print "FIELDS main=$mainv daily=$dailyv bytecode=$bytecodev\n";

$ENV{'LD_LIBRARY_PATH'} = $ium;

updateFile('main',$mainv);
updateFile('daily',$dailyv);
updateFile('bytecode',$bytecodev);

rmdir("$clamdb/temp");

open V, ">>$clamdb/version.txt";
print V "clamav=$clamv,$dailyv\n";
close V;

exit $rc;

sub getTXT2 {
  my $server = shift @_;
  my $temp = "/tmp/dns.txt";
  unlink $temp if $temp;
  my $url = "http://$server";
  print `$WGET -q -O $temp $url/dns.txt 2>&1`;
  if ( -f $temp ) {
    open T, $temp;
    my $str = <T>;
    close T;
    unlink $temp;
    return $str;
  } else {
    warn "Unable to get dns.txt file\n";
    return 0;
  }
}

sub getLocalVersion {
  my $file=shift @_;
  my $cmd="$ium/sigtool -i $clamdb/$file.cvd";
  open P, "$cmd |" || die("Can't run $cmd : $!");
  while (<P>) {
    next unless /Version: (\d+)/;
    return $1;
  }
  return -1;
}

sub updateFile {
  my $file=shift @_;
  my $currentversion=shift @_;
  my $old=0;
  if  ( ! -e "$file.cvd" ) {
    warn "file $file.cvd does not exists, skipping\n";
  }
  if  ( ! -z "$file.cvd" ) {
    $old = getLocalVersion($file);
    if ( $old > 0 ) {
      print "$file old: $old current: $currentversion\n";
      # mirror all the diffs
      for (my $count = $old + 1 ; $count <= $currentversion; $count++) {
          print `$WGET -nH -nd -N -nv $mirror/$file-$count.cdiff 2>&1`;
      } 
    } else {
      warn "file $file.cvd version unknown, skipping cdiffs\n";
      $rc = 1;
    }
  } else {
    warn "file $file.cvd is zero, skipping cdiffs\n";
    $rc = 1;
  }

  return if ( $currentversion == $old );

  # update the full file using a copy, then move back
  print `cp -v -a $file.cvd temp/$file.cvd 2>&1`;
  print `cd temp && $WGET -nH -nd -N -nv $mirror/$file.cvd 2>&1`;
  if  ( -e "temp/$file.cvd" && ! -z "temp/$file.cvd" ) {
    if ( (stat("temp/$file.cvd"))[9] > (stat("$file.cvd"))[9] ) {
      print "file temp/$file.cvd is newer than $file.cvd\n";
      print `mv -v temp/$file.cvd $file.cvd 2>&1`;
    } else {
      print "file temp/$file.cvd not touched by wget\n";
      print `rm -v temp/$file.cvd`;
    }
  } else {
    warn "temp/$file.cvd is not valid, not copying back !\n";
    unlink "temp/$file.cvd";
    $rc = 1;
  }
}
