#!/bin/sh
MANAGER_DIR=/var/www/manager
DOWNLOAD_DIR=$MANAGER_DIR/data/download
WGET=/usr/bin/wget
TEMP_DIR=/tmp/.psmt2_mirror

for PARAM in "$@"
do
	case ${PARAM} in
		--update-servers=*)
			update_servers=`echo $PARAM | cut -d '=' -f2 | sed -e 's/,/ /g'`
			;;

		--nodns)
			dns_resolv=no
			;;

		--proxy-host=*)
			PROXY_HOST=`echo $PARAM | cut -d '=' -f2`
			if [ "$PROXY_HOST" = "" ]
			then
				echo "proxy host failed"
				exit 1
			fi
			;;

		--proxy-port=*)
			PROXY_PORT=`echo $PARAM | cut -d '=' -f2`
			if [ "$PROXY_PORT" = "" ]
			then
				echo "proxy port failed"
				exit 1
			fi
			;;

		--proxy-user=*)
			PROXY_USER=`echo $PARAM | cut -d '=' -f2`
			if [ "$PROXY_USER" = "" ]
			then
				echo "proxy user failed"
				exit 1
			fi
			;;

		--proxy-password=*)
			PROXY_PASS=`echo $PARAM | cut -d '=' -f2`
			if [ "$PROXY_PASS" = "" ]
			then
				echo "proxy password failed"
				exit 1
			fi
			;;

		*)
			;;
	esac
done

#default setting
if [ "$update_servers" = "" ]
then
    update_servers="clamav-latest.promark-inc.com clamav-latest-2.promark-inc.com"
fi

#proxy setting
if [ "$PROXY_HOST" != "" ]
then
    http_proxy="http://$PROXY_HOST"
    if [ "$PROXY_PORT" != "" ]
    then
	http_proxy="$http_proxy:$PROXY_PORT"
    fi
    if [ "$PROXY_USER" != "" ]
    then
	WGET="$WGET --proxy-user=$PROXY_USER --proxy-password=$PROXY_PASS"
    fi
    export http_proxy
fi

# main logic

# make temporary directory
if [ -d $TEMP_DIR ]
then
    rm -r $TEMP_DIR > /dev/null 2>&1
fi
mkdir $TEMP_DIR > /dev/null 2>&1

# get update list
for server in $update_servers
do
    echo -n "get PSMT agent packages from $server ... "
    $WGET -q -O $TEMP_DIR/psmt-agent-pkg.tar.gz http://$server/psmt2/psmt-agent-pkg.tar.gz
    flag1=$?
    if [ $flag1 -eq 0 ]
    then
	echo "successful"
        break
    fi
    echo "failed"
done

echo -n "get PSMT manager packages from $server ... "
$WGET -q -O $TEMP_DIR/psmt-manager-pkg.tar.gz http://$server/psmt2/psmt-manager-pkg.tar.gz
flag2=$?
if [ $flag2 -eq 0 ]
then
    echo "successful"
else
    echo "failed"
fi

if [ $flag1 -ne 0 -o $flag2 -ne 0 ]
then
    echo "PSMT mirror failed. PSMT package could not download."
    exit 1
fi

cp $TEMP_DIR/psmt-agent-pkg.tar.gz $DOWNLOAD_DIR/agent_pkg.tar.gz
cp $TEMP_DIR/psmt-manager-pkg.tar.gz $MANAGER_DIR/manager_pkg.tar.gz
 
rm -r $TEMP_DIR

exit 0
